/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import java.util.*;

/**
 *
 * @author Dmitry
 */
class Agent {
    private ArrayList<Enrichment> enrichments;
    private ArrayList<Operator> operators;
    private ArrayList<Mining> minings;
    private ArrayList<MoxSupplier> mox_suppliers;
    private ArrayList<RemixSupplier> remix_suppliers;
    private final String name;
    private TradeBulletin bulletin;
    private Region region;

    
    private HashMap<String, HashMap<Integer,Double>> relations = new HashMap<>();
    private HashMap<Short, HashMap<Integer, HashMap<String,Double>>> logistics = new HashMap<>();
    private HashMap<Short, HashMap<String, HashMap<Integer,Double>>> quotas = new HashMap<>();
    
    private HashMap<String, HashMap<Integer,Double>> readed_relations;
    private HashMap<Short, HashMap<Integer, HashMap<String,Double>>> readed_logistics;
    private HashMap<Short, HashMap<String, HashMap<Integer,Double>>> readed_quotas;
    
    static final double DEFAULT_RELATIONS = 5;    
    static final double DEFAULT_QUOTAS = Double.POSITIVE_INFINITY; 
    static final double DEFAULT_LOGISTIC = 0.0; 

    public Agent(String name) {
        this.enrichments = new ArrayList<>();
        this.operators = new ArrayList<>();
        this.minings = new ArrayList<>();
        this.mox_suppliers = new ArrayList<>();
        this.remix_suppliers = new ArrayList<>();
        this.name = name;
        
    }
    
    public Agent(String name, Region region) {
        this.enrichments = new ArrayList<>();
        this.operators = new ArrayList<>();
        this.minings = new ArrayList<>();
        this.mox_suppliers = new ArrayList<>();
        this.remix_suppliers = new ArrayList<>();
        this.name = name;
        this.region = region;
    }
    
    public void initRelations(ArrayList<String> agents_names){
        for (int i=0; i < agents_names.size(); i++){          
            for (int j=0; j < Time.years.length; j++){
                if (agents_names.get(i).equals(this.name)){
                    this.setRelation(agents_names.get(i), Time.years[j], 10.0);
                }
                else{
                    this.setRelation(agents_names.get(i), Time.years[j], DEFAULT_RELATIONS);
                }
            }                                           
        }
    } 
    
    public void initQuotas(ArrayList<String> agents_names){
        Short[] prods = {Contract.P_OUP, Contract.P_PK, Contract.P_SWU, Contract.P_MOX};

        for (int i=0; i < agents_names.size(); i++){          
            for (int j=0; j < Time.years.length; j++){
                for (int k=0; k < prods.length; k++){
                    this.setQuota(prods[k], agents_names.get(i), Time.years[j], DEFAULT_QUOTAS);
                }
            }                                           
        }
    } 
    

    
    public void initLogistics(ArrayList<String> agents_names){
        Short[] prods = {Contract.P_OUP, Contract.P_PK, Contract.P_SWU, Contract.P_MOX};

        for (int i=0; i < Time.years.length; i++){          
            for (int j=0; j < agents_names.size(); j++){
                for (int k=0; k < prods.length; k++){
                    this.setLogistic(prods[k], agents_names.get(j), Time.years[i], DEFAULT_LOGISTIC);
                }
            }                                           
        }
    } 
    
    public void setReadedRelations(HashMap<String, HashMap<Integer, Double>> relations) {
        this.readed_relations = relations;
    }
    
    public void setReadedQuotas(HashMap<Short, HashMap<String, HashMap<Integer,Double>>> q) {
        this.readed_quotas = q;
    }
        
    public void setReadedLogistics(HashMap<Short, HashMap<Integer, HashMap<String,Double>>> log) {
        this.readed_logistics = log;
    }
    
    public void setRelations(HashMap<String, HashMap<Integer, Double>> Relations) {
        this.relations = Relations;
    }
    
    
    public void setLogistic(short p_id, String agentName, int year, double rel){
        if (this.logistics == null){
            this.logistics = new HashMap<>();
            System.out.println("Agent Logistic was null!!!");
        }
        
        if (logistics.containsKey(p_id)){
            HashMap<Integer, HashMap<String, Double>> yearHash = this.logistics.get(p_id);
            if (yearHash.containsKey(year)){
                HashMap<String, Double> agHash = yearHash.get(year);
                if (agHash.containsKey(agentName)){
                    agHash.replace(agentName, rel);
                }else{
                    agHash.put(agentName, rel);
                }
            }else{
                HashMap<String, Double> newYear = new HashMap<String,Double>();
                newYear.put(agentName, rel);
                yearHash.put(year, newYear);
            }
        }
        else{
            HashMap<String, Double> newAgent = new HashMap<String,Double>();
            newAgent.put(agentName, rel);
            
            HashMap<Integer, HashMap<String, Double>> newYear = new HashMap<>();
            newYear.put(year, newAgent);
                  
            this.logistics.put(p_id, newYear);
        }
    }
    
    public void setQuota(short p_id, String agentName, int year, double quota){
        if (this.quotas == null){
            this.quotas = new HashMap<>();
            System.out.println("Agent Quotas was null!!!");
        }
        double recalc_quota;
        if (operators.isEmpty()){
            if (! enrichments.isEmpty()){
                if (p_id == Contract.P_PK)
                   { 
                    recalc_quota = Enrichment.calc_clever_pk_quotas(enrichments, quota, year);
                }
                else{
                   recalc_quota = quota; 
                }
            } else{
                recalc_quota = quota; 
            }
        }else{
           if (p_id == Contract.P_OUP){ 
                recalc_quota = Operator.calc_clever_quotas(operators, quota, year);
           } 
           else if (p_id == Contract.P_SWU)
               { 
                recalc_quota = Operator.calc_clever_swu_quotas(operators, quota, year);
           }
           else if (p_id == Contract.P_PK)
               { 
                recalc_quota = Operator.calc_clever_pk_quotas(operators, quota, year);
           }
           else{
               recalc_quota = quota; 
           }
        }
        if (quotas.containsKey(p_id)){
            HashMap<String, HashMap<Integer, Double>> agHash = this.quotas.get(p_id);
            if (agHash.containsKey(agentName)){
                HashMap<Integer, Double> yearHash = agHash.get(agentName);
                if (yearHash.containsKey(year)){
                    yearHash.replace(year, recalc_quota);
                }else{
                    yearHash.put(year, recalc_quota);
                }
            }else{
                HashMap<Integer, Double> newAgent = new HashMap<Integer,Double>();
                newAgent.put(year, recalc_quota);
                agHash.put(agentName, newAgent);
            }
        }
        else{
            HashMap<Integer, Double> newAgent = new HashMap<Integer,Double>();
            newAgent.put(year, recalc_quota);

            HashMap<String, HashMap<Integer, Double>> newProd = new HashMap<>();
            newProd.put(agentName, newAgent);

            this.quotas.put(p_id, newProd);
        }
    }
    
    public void setRelation(String AgentName, int Year, double rel){
        if (this.relations == null){
            this.relations = new HashMap<>();
            //System.out.println("Agent relations was null!!!");
        }
                
        if (relations.containsKey(AgentName)){
            HashMap<Integer, Double> AgentHash = relations.get(AgentName);
            if (AgentHash.containsKey(Year)){
                AgentHash.replace(Year, rel);
            }else{
                AgentHash.put(Year, rel);
            }
        }else{
            HashMap<Integer, Double> newAgent = new HashMap<Integer,Double>();
            newAgent.put(Year, rel);
            relations.put(AgentName, newAgent);
        }
    }

    public void setLogistic1(HashMap<Short, HashMap<Integer, HashMap<String,Double>>> original){
        for (Map.Entry<Short, HashMap<Integer, HashMap<String,Double>>> product_entry : original.entrySet()){
            HashMap<Integer, HashMap<String,Double>> copypart_by_product = new HashMap<>();
            for (Map.Entry<Integer, HashMap<String,Double>> entry : product_entry.getValue().entrySet()){
                HashMap<String,Double> copypart = new HashMap<String,Double>();
                    for (Map.Entry<String,Double> entrypart : entry.getValue().entrySet()){
                        copypart.put(entrypart.getKey(), entrypart.getValue());
                    }
                copypart_by_product.put(entry.getKey(), copypart);  
                this.logistics.replace(product_entry.getKey(),copypart_by_product);
            }
        }
    }
    
    public void setQuota(HashMap<Short, HashMap<String, HashMap<Integer,Double>>> original){
        for (Map.Entry<Short, HashMap<String, HashMap<Integer,Double>>> product_entry : original.entrySet()){
            HashMap<String, HashMap<Integer,Double>> copypart_by_product = new HashMap<>();
            for (Map.Entry<String, HashMap<Integer,Double>> entry : product_entry.getValue().entrySet()){
                HashMap<Integer,Double> copypart = new HashMap<Integer,Double>();
                    for (Map.Entry<Integer,Double> entrypart : entry.getValue().entrySet()){
                        copypart.put(entrypart.getKey(), entrypart.getValue());
                    }
                copypart_by_product.put(entry.getKey(),copypart);
                this.quotas.replace(product_entry.getKey(),copypart_by_product);
            }
        }
    }
    
    public void setRelation(HashMap<String, HashMap<Integer,Double>> original){
        for (Map.Entry<String, HashMap<Integer,Double>> entry : original.entrySet()){
            HashMap<Integer,Double> copypart = new HashMap<Integer,Double>();
                for (Map.Entry<Integer,Double> entrypart : entry.getValue().entrySet()){
                    copypart.put(entrypart.getKey(), entrypart.getValue());
                }
            this.relations.replace(entry.getKey(),copypart);
        }
    }    
    
    double getRelation(String seller, int yTime) {
        return relations.get(seller).get(yTime);
    }
    
        
    public Region getRegion() {
        return region;
    }
    
    public HashMap<String, HashMap<Integer, Double>> getReadedRelations() {
        return this.readed_relations ;
    }
    
    public HashMap<Short, HashMap<String, HashMap<Integer,Double>>> getReadedQuotas() {
        return this.readed_quotas;
    }
        
    public HashMap<Short, HashMap<Integer, HashMap<String,Double>>> getReadedLogistics() {
        return this.readed_logistics;
    }
    
    public void add_actor(Actor actor){
        //Class<? extends Actor> cc = actor.getClass();
        //cc.
        String c = actor.getClass().getSimpleName();
    switch (c){
        case "Enrichment": enrichments.add((Enrichment) actor);break;
        case "TVEL": enrichments.add((Enrichment) actor);break;
        case "Operator": operators.add((Operator) actor);break;
        case "Mining": minings.add((Mining) actor);break;
        case "MoxSupplier": mox_suppliers.add((MoxSupplier) actor);break;
        case "Trader": minings.add((Mining) actor);break;
        case "PKDump": minings.add((Mining) actor);break;
        case "RemixSupplier": remix_suppliers.add((RemixSupplier) actor);break;
        default:  throw new RuntimeException("Unexpected Class"); //To change body of generated methods, choose Tools | Templates.();
    }
    
    }
    public void set_bulletin( TradeBulletin tbulletin){
        this.bulletin = tbulletin;
    }

    ArrayList<MoxSupplier> get_mox_supplier() {
        return this.bulletin.my_mox_suppliers;
    }
    ArrayList<Enrichment> get_eup_supplier() {
        return this.bulletin.my_enrichments;
    }
    ArrayList<Enrichment> get_swu_supplier() {
        return this.bulletin.my_enrichments;
    }
    
    ArrayList<Enrichment> get_eup_supplier(double need) {
        ArrayList<Enrichment> all_suppliers = this.bulletin.my_enrichments;
        ArrayList<Enrichment> my_suppliers = new ArrayList();
        all_suppliers.stream().filter((supplier) -> (quotas.get(Contract.P_OUP).get(supplier.getAgent().getName()).get(Time.yTime) > need)).forEachOrdered((supplier) -> {
            my_suppliers.add(supplier);
        });
        return my_suppliers;
    }
    
    ArrayList<Enrichment> get_rus_eup_supplier() {
        ArrayList<Enrichment> all_suppliers = this.bulletin.my_enrichments;
        ArrayList<Enrichment> my_suppliers = new ArrayList();
        all_suppliers.stream().filter((supplier) -> supplier.getRegion().getName().equals("Russia")).forEachOrdered((supplier) ->  my_suppliers.add(supplier));
         return my_suppliers;
    }
    
    ArrayList<Mining> get_min_supplier() {
      return this.bulletin.my_minings;
    }
    
    ArrayList<Mining> get_min_supplier(double need) {
       ArrayList<Mining> all_suppliers = this.bulletin.my_minings;
        ArrayList<Mining> my_suppliers = new ArrayList();
        all_suppliers.stream().filter((supplier) -> (quotas.get(Contract.P_PK).get(supplier.getAgent().getName()).get(Time.yTime) > need)).forEachOrdered((supplier) -> {
            my_suppliers.add(supplier);
        });
        my_suppliers.removeIf(S -> (S.getClass().equals(Trader.class) && Time.yTime  < S.getActive_year()) );
       /* boolean[] V = new boolean[my_suppliers.size()];
        boolean[] V1 = new boolean[my_suppliers.size()];
        boolean[] V2 = new boolean[my_suppliers.size()];
        int T;
        int i =0;  
        for (Mining S : my_suppliers) {
            V[i] = (S.getClass().equals(Trader.class) &&  S.getActive_year() < Time.yTime);
            V1[i] = S.getClass().equals(Trader.class);
            V2[i] = Time.yTime  < S.getActive_year();
            T = S.getActive_year();
            i++;
        }*/
        return my_suppliers; 
    }
    
    ArrayList<Enrichment> get_swu_supplier(double need) {
        ArrayList<Enrichment> all_suppliers = this.bulletin.my_enrichments;
        ArrayList<Enrichment> my_suppliers = new ArrayList();
        all_suppliers.stream().filter((supplier) -> (quotas.get(Contract.P_SWU).get(supplier.getAgent().getName()).get(Time.yTime) > need)).forEachOrdered((supplier) -> {
            my_suppliers.add(supplier);
        });
        return my_suppliers;
    }
    
    RemixSupplier get_remix_supplier() {
        return this.bulletin.remix_supplier;
    }
    
    String getName(){
        return this.name;
    }

    HashMap<String, HashMap<Integer,Double>> getRelations() {
        return this.relations;
    }
    
    HashMap<Short, HashMap<Integer, HashMap<String,Double>>> getLogistics1() {
        return this.logistics;
    }
    HashMap<String, Double> getLogistics(short P, int year) {
        return logistics.get(P).get(year);
    }
    double getLogistics(short P, String name, int year) {
        return logistics.get(P).get(year).get(name);
    }
    HashMap<String, HashMap<Integer,Double>> getRelations1() {
        return this.relations;
    }

    HashMap<Short, HashMap<String, HashMap<Integer,Double>>> getQuotas1() {
        return this.quotas;
    }

    public ArrayList<Enrichment> get_enrichments(){
        return this.enrichments;
    }
    
    public ArrayList<Operator> get_operators(){
        return operators;
    }
    
    public ArrayList<Mining> get_minings(){
        return this.minings;
    }
    
    public ArrayList<MoxSupplier> get_mox_suppliers(){
        return this.mox_suppliers;
    }

    boolean fail_quota(Contract W) {
        boolean answer = false;
        switch (W.product){
            case Contract.P_MOX: {
                double A = 0;
                for (Operator operator : operators) {
                    A = A+ operator.stats.mox.get(W.year).get(W.get_seller().getName());
                }
                answer = quotas.get(Contract.P_MOX).get(W.get_seller().getAgent().getName()).get(W.year) < (A + W.volume);
                break;
            }
            case Contract.P_REMIX: {
                double A = 0;
                for (Operator operator : operators) {
                    A = A+ operator.stats.remix.get(W.year).get(W.get_seller().getName());
                }
                answer = quotas.get(Contract.P_REMIX).get(W.get_seller().getAgent().getName()).get(W.year) < (A + W.volume);
                break;
            }
            case Contract.P_OUP: {
                double A = 0;
                for (Operator operator : operators) {
                    A = A+ operator.stats.eup.get(W.year).get(W.get_seller().getName());
                }
                answer = quotas.get(Contract.P_OUP).get(W.get_seller().getAgent().getName()).get(W.year) < (A + W.volume);
                break;
            }
            case Contract.P_PK: {
                double A = 0;
                for (Operator operator : operators) {
                    A = A+ operator.stats.pk.get(W.year).get(W.get_seller().getName());
                }
                for (Enrichment enrichment : enrichments) {
                    A = A+ enrichment.stats.pk.get(W.year).get(W.get_seller().getName());
                }
                answer = quotas.get(Contract.P_PK).get(W.get_seller().getAgent().getName()).get(W.year) < (A + W.volume);
                break;
            }
            default: throw new Error("Undefined type");
        }
        return answer;
    }
    
    
    boolean fail_quota(ContractSWUPK W){
        boolean answer;
        double A = 0;
        for (Operator operator : operators) {
            A = A+ operator.stats.pk.get(W.year).get(W.getPk_seller().getName());
        }
        for (Enrichment enrichment : enrichments) {
            A = A+ enrichment.stats.pk.get(W.year).get(W.getPk_seller().getName());
        }
        boolean  answer1 = quotas.get(Contract.P_PK).get(W.getPk_seller().getAgent().getName()).get(W.year) < (A + W.pk_volume);
        
        double B = 0;
        for (Operator operator : operators) {
            B = B + operator.stats.swu.get(W.year).get(W.getEn_seller().getName());
        }
        boolean answer2 = quotas.get(Contract.P_SWU).get(W.getEn_seller().getAgent().getName()).get(W.year) < (B + W.en_volume);
        answer = answer2 || answer1;
        return answer; 
    }









    
}
