/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.OptionalDouble;

/**
 *
 * @author Dmitry
 */
class Enrichment extends Producer {

    static double calc_pk_volume(double volume, double enrichment) {
        
        return volume*(enrichment - World.waste_enrichment)/(World.natural_enrichment - World.waste_enrichment); 
    }

    static double calc_waste_volume(double volume, double enrichment) {
        return volume*(enrichment - World.natural_enrichment)/(World.natural_enrichment - World.waste_enrichment);
    }

    static double calc_swu(double PK, double Waste, double volume, double enrichment) {
        return volume*Enrichment.separation_potential(enrichment) - 
                PK*Enrichment.separation_potential(World.natural_enrichment) + 
                Waste*Enrichment.separation_potential(World.waste_enrichment);
    }
    
    static double calc_swu( double volume, double enrichment) {
        double PK = Enrichment.calc_pk_volume(volume, enrichment);
        double Waste = Enrichment.calc_waste_volume(volume, enrichment);
        return volume*Enrichment.separation_potential(enrichment) - 
                PK*Enrichment.separation_potential(World.natural_enrichment) + 
                Waste*Enrichment.separation_potential(World.waste_enrichment);
    }
    
    static double calc_swu( double volume, double enrichment, double tail) {
        double PK = Enrichment.calc_pk_volume(volume, enrichment);
        double Waste = Enrichment.calc_waste_volume(volume, enrichment, tail);
        return volume*Enrichment.separation_potential(enrichment) - 
                PK*Enrichment.separation_potential(World.natural_enrichment) + 
                Waste*Enrichment.separation_potential(tail);
    }
    
    static double calc_waste_volume(double volume, double enrichment,  double tail) {
        return volume*(enrichment - World.natural_enrichment)/(World.natural_enrichment - tail);
    }
    
    static double calc_pk_volume(double volume, double enrichment,  double tail) {
        
        return volume*(enrichment - tail)/(World.natural_enrichment - tail); 
    }
    
    private static double separation_potential(double X) {
        return (2*X - 1)*Math.log(X/(1-X));
    }

    static double calc_clever_pk_quotas(ArrayList<Enrichment> enrichments, double quota, int year) {
        double mean_X = 0.045;
        if (quota == 0)
            return 0;
        if (quota == Double.POSITIVE_INFINITY)
            return Double.POSITIVE_INFINITY;
        ArrayList<Double> needsList = new ArrayList<>();
        for (Enrichment enrichment : enrichments) {
            double F = enrichment.hipp_capacity*(mean_X - enrichment.tail)/(World.natural_enrichment - enrichment.tail);
            needsList.add(F);
        }
        Double[] needs = new Double[needsList.size()];
        needs = needsList.toArray(needs);
        double sum_needs = DMath.Arithmetic.sum(needs);
        if (sum_needs == 0)
            return 0;    
        return sum_needs*quota;
    }

    double tail = 0.0016;
    double storage = 0;
    double[] opt_tails;
    double[] prod_tails;
    double investment_cost = 10;
    double hipp_capacity;
    double[] hipp_capacities;
    PKRaitingFuzzyCalculator PKFuzzyCalculator;
    EupMargeFuzzyCalculator MargeFuzzyCalculator; 
    EupDiscountFuzzyCalculator DiscountFuzzyCalculator; 
    double[] F_needs;
    double[] storages;
    double[] F_Givens;
    double F;
    public Enrichment() {
        PKFuzzyCalculator = new PKRaitingFuzzyCalculator(this);
    }

    public Enrichment(double capacity, double cost, double[] all_capacity, double[] all_cost) {
        super();
        this.capacity = capacity;
        hipp_capacities = new double[Time.years.length];
        F_needs = new double[Time.years.length];
        F_Givens = new double[Time.years.length];
        storages = new double[Time.years.length];
        calc_hipp_capacity();
        hipp_capacities[0] = hipp_capacity;
        this.cost = cost;
        this.all_capacity = all_capacity;
        this.all_cost = all_cost;
        opt_tails = new double[Time.years.length];
        opt_tails[0] = tail;
        prod_tails = new double[Time.years.length];
        PKFuzzyCalculator = new PKRaitingFuzzyCalculator(this);
        MargeFuzzyCalculator = new EupMargeFuzzyCalculator(this);
        DiscountFuzzyCalculator = new EupDiscountFuzzyCalculator(this);
    }
    
    void calc_hipp_capacity(){
        double y_max = 0.004;
        double x_mean = 0.045;
        
        hipp_capacity = capacity/(separation_potential(x_mean) - (x_mean - y_max)/(World.natural_enrichment - y_max)*
                separation_potential(World.natural_enrichment) + (x_mean - World.natural_enrichment)/(World.natural_enrichment - y_max)*
                separation_potential(y_max ) );
    }
    
    public double getInvestmentCost(){
        return this.investment_cost;
    }
    
    public  double getTail(){
        return this.tail;
    }
    
    public void setInvestmentCost(double ic){
        this.investment_cost = ic;
    }
    
    public void setTail(double tail){
        this.tail = tail;
    }
    
    @Override
    double calc_pk(Contract c) {
        return Enrichment.calc_pk_volume(c.volume, c.enrichment, tail);
    }

    @Override
    double calc_swu(Contract c) {
        return Enrichment.calc_swu(c.volume, c.enrichment, tail);
    }

 

    @Override
    boolean is_capable(Contract c) {
       // double volume = Enrichment.calc_swu(c.volume, c.enrichment);
        return hipp_capacity >= c.volume; 
    }

    boolean is_capable(Contract c, boolean is_piece) {
       // double volume = Enrichment.calc_swu(c.volume, c.enrichment);
       if (is_piece)
            return hipp_capacity >= c.volume;
       else
           return hipp_capacity >= c.volume;
            //return capacity >= c.en_volume;
    }
    
    @Override
    double calc_prioritet(Contract c) {
        return PKFuzzyCalculator.calc_prioritet(c);
    }

    @Override
    double calc_marge(Contract c) {
        double marge = MargeFuzzyCalculator.calc_marge(c);
        c.setMarge(marge);
        return marge;
    }
    @Override
    double calc_marge(ContractSWUPK c) {
        double marge = MargeFuzzyCalculator.calc_marge(c);
        c.setEnMarge(marge);
        return marge;
    }
    @Override
    double calc_discount(Contract c) {
        return DiscountFuzzyCalculator.calc_discount(c); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    double set_discount_price(ContractSWUPK c_swupk) {
        double en_discount = calc_discount(c_swupk);
        //double new_en_price = cost*(1 +c_swupk.en_marge- en_discount);
        double new_en_price = c_swupk.en_price*(1-en_discount);
        if (new_en_price < c_swupk.min_en_price){new_en_price = c_swupk.min_en_price;}   
        return new_en_price;
    }
    
    @Override
    double set_discount_price(Contract c, double mean_pk_price){
         double new_price;
         double discount = c.get_seller().calc_discount(c);
         double swu_volume = Enrichment.calc_swu(c.volume, c.enrichment, tail);
         double pk_vol = c.getSon(0).volume;
         double SWUTC;
        if (pbase == PBASE_COST){
            double net_cost = calc_net_cost(c, mean_pk_price);
            new_price = (pk_vol*mean_pk_price + swu_volume*(this.cost*(1+c.marge)*(1-discount)+investment_cost))/c.volume;
            c.setMin_price((pk_vol*mean_pk_price + swu_volume*(this.cost*(1+min_marge)+investment_cost))/c.volume);
        }
        else{
            new_price = c.price*(1-discount);
            }
        if (new_price < c.min_price)
            {new_price = c.min_price;}
        SWUTC = new_price - pk_vol*mean_pk_price;
        c.setSWU_stat(SWUTC/swu_volume, swu_volume);
        return new_price;     
     }
    
    
    @Override
    double calc_price(Contract c, double mean_pk_price) {
        double marge = calc_marge(c);
        c.set_marge_in_tender(marge);
        double swu_volume = Enrichment.calc_swu(c.volume, c.enrichment, tail);
        double pk_vol = c.getSon(0).volume;
        if (pbase == PBASE_COST){
            c.setMin_price((pk_vol*mean_pk_price + swu_volume*(this.cost*(1+min_marge)+investment_cost))/c.volume);
            double delta = World.spotSWUprice[Time.iTime] - this.cost*(1+min_marge);
            double enrich_price;
            
            double m1;
            if (delta > 0 ){
                m1 = 1 - Math.min(1, delta/cost);
                enrich_price = swu_volume*(this.cost*(1+marge)+investment_cost + delta*(1+marge*m1)) ;
                }
            else
                enrich_price = swu_volume*(this.cost*(1+marge)+investment_cost);
            c.setSWU_stat(enrich_price/swu_volume,swu_volume);
            double price = (pk_vol*mean_pk_price + enrich_price)/c.volume;
            return price; 
        }else{
            c.setMin_price(World.spotEUPprice[Time.iTime]*0.8);
            double TP = c.volume* (1+marge)*World.spotEUPprice[Time.iTime];
            double SWU_cost = TP - pk_vol*mean_pk_price;
            c.setSWU_stat(SWU_cost/swu_volume, swu_volume);
            return (1+marge)*World.spotEUPprice[Time.iTime];
        }
    }
    
    @Override
    double calc_price(ContractSWUPK c){
        double marge = calc_marge(c);
        if (pbase == PBASE_COST){
            double delta = World.spotSWUprice[Time.iTime] - this.cost*(1+min_marge);
            double price;
            double m1;
            if (delta > 0 ){
                m1 = 1 - Math.min(1, delta/cost);
                price = cost*(1+marge)+investment_cost + delta*(1+marge*m1);
                }
            else
                price = (cost)*(1+marge)+investment_cost;
            c.setMin_en_price((cost)*(1+min_marge)+investment_cost);
            return price;
        }else{
            double price = World.spotSWUprice[Time.iTime]*(1+marge);
            c.setMin_en_price(World.spotSWUprice[Time.iTime]*0.8);
            return price;
        }
    }

    @Override
    double calc_net_cost(Contract c, double mean_pk_price) {
        double pk_vol = c.getSon(0).volume;
        double net_cost = (pk_vol*mean_pk_price + Enrichment.calc_swu(c.volume, c.enrichment, tail)*this.cost)/c.volume;
        return net_cost;
    }
    
    double calc_net_cost(Contract c, double mean_pk_price, double PK) {
        double pk_vol = PK;
        double net_cost = (pk_vol*mean_pk_price + Enrichment.calc_swu(c.volume, c.enrichment, tail)*this.cost)/c.volume;
        return net_cost;
    }
    
    @Override
    void deal_pk(Contract c) {
        this.money = this.money - c.price*c.volume; 
        Agreement aa = Agreement.create_agreement(c);
        F = F - c.volume; 
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.price*c.volume;
    }

    @Override
    void deal_eup(Contract c) {
        money = money + c.price*c.volume;
        double cap_volume = Enrichment.calc_swu(c.volume, c.enrichment,tail);
        double cap_volume2 = this.calc_swu(c);
        capacity = capacity - cap_volume;
        hipp_capacity = hipp_capacity - c.volume;
        Agreement aa = Agreement.create_agreement(c);
        sales.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        account_waste(c);
        //stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.en_volume*cost;
        stats.revenue[Time.iTime] =  stats.revenue[Time.iTime] + c.price*c.volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.en_volume;
    }

    @Override
    void deal_swu_pk(ContractSWUPK c) {
        money = money + c.en_price*c.en_volume;
        capacity = capacity - c.en_volume;
        hipp_capacity = hipp_capacity - c.volume;
        Agreement[] aa = Agreement.create_agreement(c);
        sales.get(c.year).add(aa[1]);
        stats.recalc_deals_stat(c.year, aa[1].product);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.en_volume*cost;
        stats.revenue[Time.iTime] =  stats.revenue[Time.iTime] + c.en_price*c.en_volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.en_volume;
        account_waste(c);
    }
    
    void deal_swu(Contract c) {
        money = money + c.price*c.volume;
        capacity = capacity - c.en_volume;
        Agreement aa = Agreement.create_agreement(c);
        sales.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, aa.product);
        //stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.volume*cost;
        stats.revenue[Time.iTime] =  stats.revenue[Time.iTime] + c.price*c.volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.en_volume;
        account_waste(c);
    }
    private void account_waste(Contract c){
        double waste = calc_waste_volume(c.volume, c.enrichment, tail);
        int f = DMath.Arithmetic.indexOf(Time.years, c.year);
        stats.deplited[f] = stats.deplited[f]+waste;
    }   

    double get_min_price() {
        return cost * (1 + min_marge);
    }

    @Override
    void restore() {
        super.restore();
        calc_hipp_capacity();
        if(Time.iTime < (Time.lastTime-1)){
            hipp_capacities[Time.iTime+1] = hipp_capacity;
            calc_opt_tail();
            check_pbase();
        }
    }
    
    private void check_pbase(){
        if (World.Switch_PBase && ( id!=152)){
            double Spot;
            if (Time.iTime > 2){
                Spot = World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)]*0.9/(0.9+0.8+0.7) + 
                       World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime)]*0.8/(0.9+0.8+0.7) +
                       World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.7/(0.9+0.8+0.7);
            }
            else
                Spot = World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)];
            
            if (Spot*0.95 > cost*1.05){
                pbase = Producer.PBASE_COST;
            } else{
                pbase = Producer.PBASE_PRICE;
            }
        }
    }

    @Override
    double calc_opt_tail_price(Contract c, double mean_pk_price) {
        double marge = calc_marge(c);
        c.set_marge_in_tender(marge);
        double swu_volume = Enrichment.calc_swu(c.volume, c.enrichment, tail);
        double pk_vol = calc_pk_volume(c.volume, c.enrichment, tail);
        double SWU_cost;
        if (pbase == PBASE_COST){
            //double costl = (pk_vol*mean_pk_price + Enrichment.calc_swu(c.volume, c.enrichment, tail)*this.cost)/c.volume;
            c.setMin_price((pk_vol*mean_pk_price + swu_volume*(this.cost*(1+min_marge)+investment_cost))/c.volume);
            double price = (pk_vol*mean_pk_price + swu_volume*(this.cost*(1+marge)+investment_cost))/c.volume;
            SWU_cost = price*c.volume - pk_vol*mean_pk_price;
            c.setSWU_stat(SWU_cost/swu_volume, swu_volume);
            return price; 
        }else{
            double TP = c.volume* (1+marge)*World.spotEUPprice[Time.iTime];
            SWU_cost = TP - pk_vol*mean_pk_price;
            c.setSWU_stat(SWU_cost/swu_volume, swu_volume);
            
            c.setMin_price(World.spotEUPprice[Time.iTime]*0.8);
            return (1+marge)*World.spotEUPprice[Time.iTime];
        }
    }
    
    @Override
    double set_opt_tail_discount_price(Contract c, double mean_pk_price){
       // double discount = c.get_seller().calc_discount(c);
       // double new_price = c.price*(1-discount);
        
        double new_price;
        double discount = c.get_seller().calc_discount(c);
        double swu_volume = Enrichment.calc_swu(c.volume, c.enrichment, tail);
        double pk_vol = calc_pk_volume(c.volume, c.enrichment, tail);
        double SWUTC;
        if (pbase == PBASE_COST){
            new_price = (pk_vol*mean_pk_price + swu_volume*(this.cost*(1+c.marge)*(1-discount)+investment_cost))/c.volume;
            c.setMin_price((pk_vol*mean_pk_price + swu_volume*(this.cost*(1+min_marge)+investment_cost))/c.volume);
        }
        else{
            new_price = c.price*(1-discount);
            }
        
        
        if (new_price < c.min_price)
            {new_price = c.min_price;}
        
        SWUTC = new_price*c.volume - pk_vol*mean_pk_price;
        c.setSWU_stat(SWUTC/swu_volume, swu_volume);
        return new_price;     
     }
    
    void calc_opt_tail() {
        double phi = calc_RF_price();
        
        double new_tail = 0.00228*Math.exp(0.468*Math.log(phi) - 0.074*Math.pow(Math.log(phi),2));
        opt_tails[Time.iTime+1] = new_tail;
        tail = new_tail;
    }

    private double calc_RF_price() {
//        ArrayList<Agreement> this_year_pk_agr = purchases.get(Time.yTime);
//        int n = 0;
//        double volume = 0;
//        double volume_price = 0;
//        for (Agreement agreement : this_year_pk_agr) {
//            if(agreement.product == Contract.P_PK){
//                volume = volume + agreement.volume;
//                volume_price = volume_price + agreement.volume*agreement.price;
//            }
//        }
          return World.spotSWUprice[Time.iTime+1]/World.spotPKprice[Time.iTime+1];  
    }

    void buy_pk() {
        double initial_cap = all_capacity[Time.iTime];
        // Я подсчитываю свои контракты типа ЕРР и вычитаю их объем из исходной мощности
        // Если их объем больше 50% включаю их в калькуляцию хвоста
        ArrayList<Agreement> curr_year_deals = sales.get(Time.yTime);
        ArrayList<Agreement> oup_current_deals = new ArrayList();
        ArrayList<Agreement> swu_current_deals = new ArrayList();
        ArrayList<Agreement> remix_current_deals = new ArrayList();
        curr_year_deals.stream().filter(a -> a.product == Contract.P_OUP).forEachOrdered(b -> oup_current_deals.add(b));
        curr_year_deals.stream().filter(a -> (a.product == Contract.P_SWU && a.type != Contract.REMIX)).forEachOrdered(b -> swu_current_deals.add(b));
        curr_year_deals.stream().filter(a -> (a.product == Contract.P_SWU && a.type == Contract.REMIX)).forEachOrdered(b -> remix_current_deals.add(b));
        double SWU = 0;
        for (Agreement deal : swu_current_deals) {
                SWU = SWU + deal.volume;
        }
        double RSWU = 0;
        for (Agreement deal : remix_current_deals) {
                RSWU = RSWU + deal.volume;
        }
        double prod_tail;
        double F_Given = 0;
        if (SWU <= 0){
            initial_cap = initial_cap - SWU - RSWU;
            // На остаток мощности я должен распределить всю мощность
            // Решаем методом перебора от 0.0001 до 0.004
            prod_tail = calc_prod_tail(oup_current_deals,initial_cap);
        }else{
            initial_cap = initial_cap - RSWU;
            oup_current_deals.addAll(swu_current_deals);
            prod_tail = calc_prod_tail(oup_current_deals,initial_cap);
            for (Agreement swu_current_deal : swu_current_deals) {
                ContractSWUPK con =  (ContractSWUPK) swu_current_deal.contract;
                F_Given = F_Given + con.pk_volume;
            }
        }
        tail = prod_tail;
        prod_tails[Time.iTime] = prod_tail;
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  all_capacity[Time.iTime]*cost;
        // Найдем для новых хвостов новый объем ПК
        F = 0;
        for (Agreement oup_current_deal : oup_current_deals) {
            F = F + Enrichment.calc_pk_volume(oup_current_deal.eup_volume, oup_current_deal.contract.enrichment, tail);
        }
        F_needs[Time.iTime] = F;
        F_Givens[Time.iTime] = F_Given;
        F = F - F_Given - storage;
        
        if (F > 0)
            form_common_pk_contracts(F);
        else{
            storage = Math.abs(F);
            F = 0;
        }
        storages[Time.iTime] = storage;
    }

    private double calc_prod_tail(ArrayList<Agreement> current_deals, double initial_cap) {
        double initial_cap0 = all_capacity[Time.iTime];
        double[] y_points = DMath.Arithmetic.linspace(0.001, 0.004, 31);
        double[] diffR = new double[y_points.length];
        double P_sum = 0;
        for (Agreement current_deal : current_deals) {
            P_sum = P_sum + current_deal.eup_volume;
        }
        int n = 0;
        for (double y_point : y_points) {
            double R_needed = 0;
            ArrayList<Double> RRR = new ArrayList<>();
            for (Agreement oup_current_deal : current_deals) {
                double X = oup_current_deal.contract.enrichment;
                double C = World.natural_enrichment;
                double Y = y_point;
                double P = oup_current_deal.eup_volume;
                RRR.add(P*separation_potential(X));
                RRR.add(P*(X - Y)/(C-Y)*separation_potential(C));
                RRR.add(P*(X - C)/(C-Y)*separation_potential(Y));
                R_needed = R_needed + P*separation_potential(X);
                R_needed = R_needed - P*(X - Y)/(C-Y)*separation_potential(C);
                R_needed = R_needed + P*(X - C)/(C-Y)*separation_potential(Y);
            }
            diffR[n] = initial_cap - R_needed;
            n++;
        }
        OptionalDouble minOD = Arrays.stream(diffR).filter(i -> i >= 0).min();
        double min = minOD.getAsDouble();
        int I = DMath.Arithmetic.indexOf(diffR, min);
        capacity = diffR[I];
        stats.loaded[Time.iTime] = (initial_cap - capacity) + (initial_cap0 - initial_cap);
        return y_points[I];
    }

    private void form_common_pk_contracts(double FF) {
        if (FF >=0){
        double contract_val = 100;
        int I = (int) Math.floor(FF/contract_val);
            for (int i = 0; i < I; i++) {
                create_contracts(contract_val); 
            }
            if(FF%5 > 0)
                create_contracts(FF%contract_val); 
        }
    }
    
    void form_crumbs_demand(){
            double min_vol = 50;
            int I = (int) Math.floor(F/min_vol);
            for (int i = 0; i < I; i++) {
                create_contracts(min_vol); 
            }
            if(F%5 > 0)
                create_contracts(F%min_vol);
    }
    
    private void create_contracts(double f) {
        ArrayList<Mining> potential_supliers = get_my_min_supplier(f);
        ArrayList<Contract> all_created_contracts = new ArrayList(potential_supliers.size());
        for (Mining ps : potential_supliers){
            
            Contract new_contract = new Contract();
            new_contract.level = Contract.LEVEL_A;
            new_contract.type = Contract.PK_Solo;
            new_contract.volume = f;
            new_contract.product = Contract.P_PK;
            // new_contract.enrichment;
            new_contract.set_buyer(this);
            new_contract.set_seller(ps);
            new_contract.set_current_owner(ps);
            new_contract.set_reactor(new Reactor(this.getName()));
            new_contract.year = Time.yTime;
            Tender.set_tender(new_contract);
            ContractChain.add(new_contract);
            all_created_contracts.add(new_contract);
        }
        for (Contract as : all_created_contracts){
            as.set_brothers(all_created_contracts);
        }
    }
    
        @Override
    boolean is_acceptаble(Contract W) {
        int max_iter = 3;
        boolean failed_quota = check_quota(W);

        if (failed_quota){
            // Контракт должен отменяться
            W.cancel();
            W.reason = Contract.REASON_QUOTA;
            return false;
        }
        double p = W.historic_prioritet.lastElement();
        if ((W.historic_prioritet.size()) > max_iter) {
            return true;
        }
        return super.is_acceptаble(p); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    boolean isempty() {
        return all_capacity[Time.iTime] == 0; 
    }
    
    public LinkedHashMap<String, Object>  calc_region_eup_sales_stat(int year, ArrayList<Region> regions){
        LinkedHashMap<String, Object> Eimport = new LinkedHashMap<>(); 
        
        ArrayList<Agreement> my_sales = sales.get(year);
        
        for (Region region : regions) {
            ArrayList<Agreement> reg_sales = new ArrayList();
            my_sales.stream().filter(a -> a.buyer.getRegion().getName().equals(region.name)).forEachOrdered(b -> reg_sales.add(b));
            double vol = 0;
            for (Agreement reg_sale : reg_sales) {
                vol = vol + reg_sale.eup_volume;
            }
            Eimport.put(region.name, vol);
        }
 
        return Eimport;
    }
    
    public LinkedHashMap<String, Object>  calc_region_eup_sales_stat(ArrayList<Region> regions){
        LinkedHashMap<String, Object> Eimport = new LinkedHashMap<>(); 
        for (Region region : regions) {
            int i = 0;
            double[] vols = new double[Time.years.length];
            for (int year : Time.years) {
                ArrayList<Agreement> my_sales = sales.get(year);
                ArrayList<Agreement> reg_sales = new ArrayList();
                my_sales.stream().filter(a -> a.buyer.getRegion().getName().equals(region.name)).forEachOrdered(b -> reg_sales.add(b));
                double vol = 0;
                
                for (Agreement reg_sale : reg_sales) {
                    vol = vol + reg_sale.eup_volume;
                }
                vols[i] = vol;
                i++;
            }
            Eimport.put(region.name, vols);
        }
 
        return Eimport;
    }
    
    public LinkedHashMap<String, Object>  calc_region_swu_sales_stat(ArrayList<Region> regions){
        LinkedHashMap<String, Object> Eimport = new LinkedHashMap<>(); 
        for (Region region : regions) {
            int i = 0;
            double[] vols = new double[Time.years.length];
            for (int year : Time.years) {
                ArrayList<Agreement> my_sales = sales.get(year);
                ArrayList<Agreement> reg_sales = new ArrayList();
                my_sales.stream().filter(a -> a.buyer.getRegion().getName().equals(region.name)).forEachOrdered(b -> reg_sales.add(b));
                double vol = 0;
                
                for (Agreement reg_sale : reg_sales) {
                    vol = vol + reg_sale.swu_volume;
                }
                vols[i] = vol;
                i++;
            }
            Eimport.put(region.name, vols);
        }
 
        return Eimport;
    }
}
