/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Stream;

/**
 *
 * @author Dmitry
 */
public class Mining extends Producer{

    double production_rate = 0.05;
    double min_swu_marge = 0.4;
    static double upgrade_rate = 0.10;
    double[] all_reserves; 
    ArrayList<Deposit> deposits;
    ArrayList<PseudoCountry> pseudocountries;
    PKMargeFuzzyCalculator MargeFuzzyCalculator;
    PKDiscountFuzzyCalculator DiscountMargeCalculator;
    
    public Mining(double[] all_capacity, double[] all_cost, double capacity, double cost) {
        super();
        this.all_capacity = all_capacity;
        this.all_cost = all_cost;
        this.capacity = capacity;
        this.cost = cost;
        
        MargeFuzzyCalculator = new PKMargeFuzzyCalculator(this);
        DiscountMargeCalculator = new PKDiscountFuzzyCalculator(this);
    }

    public Mining(ArrayList<Deposit> deposits) {
        super();
        this.deposits = deposits;
        this.deposits.forEach(a -> a.setOwner(this));
        pseudocountries = PseudoCountry.form(deposits);
        
        capacity = 0;
        if (ScenarioTest.mining_type == ScenarioTest.IAEA_MINING){
            pseudocountries.forEach((pseudocountry) -> {
                capacity = capacity + pseudocountry.capacity;
            });
        }else{
            this.deposits.forEach(deposit -> {
                capacity = capacity + deposit.capacity;
            });
        }
        all_capacity = new double[Time.years.length];
        all_cost = new double[Time.years.length];
        calc_cost();
        for (int i = 0; i < Time.lastTime; i++) {
            all_capacity[i] = capacity;
            all_cost[i] = cost;
        }
        MargeFuzzyCalculator = new PKMargeFuzzyCalculator(this);
        DiscountMargeCalculator = new PKDiscountFuzzyCalculator(this);
        check_start_pbase();
    }
    
    public Mining(){
        super();
    }
    
    public void setDeposits(ArrayList<Deposit> deposits){
        this.deposits = deposits;
        this.deposits.forEach(a -> a.setOwner(this));
        pseudocountries = PseudoCountry.form(deposits);
        
        capacity = 0;
        double reserves = 0;
        for (Deposit deposit : deposits) {
            reserves = reserves + deposit.volume;
        }
        if (ScenarioTest.mining_type == ScenarioTest.IAEA_MINING){
            pseudocountries.forEach((pseudocountry) -> {
                capacity = capacity + pseudocountry.capacity;
                
            });
        }else{
            this.deposits.forEach(deposit -> {
                capacity = capacity + deposit.capacity;

            });
        }
        calc_cost();
        all_capacity = new double[Time.years.length];
        all_cost = new double[Time.years.length];
        all_reserves = new double[Time.years.length];
        for (int i = 0; i < Time.lastTime; i++) {
            all_capacity[i] = capacity;
            all_cost[i] = 0;
            all_reserves[i] = reserves;
        }
        MargeFuzzyCalculator = new PKMargeFuzzyCalculator(this);
        DiscountMargeCalculator = new PKDiscountFuzzyCalculator(this);
        check_start_pbase();
    }
    
    ArrayList<Deposit> getDeposits(){
        return this.deposits;
    }
    
    double[] getCosts(){
        return this.all_cost;
    }
    
    double[] getCapacities(){
        return this.all_capacity;
    }
    double calc_reserves(){
        switch(ScenarioTest.mining_type){
            case ScenarioTest.IAEA_MINING: return calc_iaea_reserves();
            case ScenarioTest.TENEX_MINING: return calc_tenex_reserves();
            default: return calc_iaea_reserves();
        }
    }
    double calc_tenex_reserves(){
        double reserves = 0;
        if (deposits.isEmpty()) return 0;
        for (Deposit depost : deposits) {
            DepositTenex depost0 = (DepositTenex) depost;
            reserves = depost0.getVolume() + reserves;
            }
        return reserves;
    }
    double calc_iaea_reserves(){
        double reserves = 0;
        if (pseudocountries.isEmpty()) return 0;
        for (PseudoCountry pseudocountry : pseudocountries) {
            reserves = reserves + pseudocountry.volume;
        }
        return reserves;
    }
    double getProductionRate(){
        return this.production_rate;
    }
    
    void setProductionRate(double rate){
        this.production_rate = rate;
    }
    
    @Override 
    double calc_price(Contract c) {
        double marge = calc_marge(c);
        if (pbase == PBASE_COST){
            double costl = calc_cost();
            double price = costl*(1+marge);
            if (c.getClass().equals(ContractSWUPK.class)){
                c.setMin_pk_price(costl, min_swu_marge);
            }else
                c.setMin_price(costl*(1+min_marge));
            return price;
        }else{
            double price = World.spotPKprice[Time.iTime]*(1+marge);
            if (c.getClass().equals(ContractSWUPK.class)){
                c.setMin_pk_price(World.spotPKprice[Time.iTime]*0.95);
            }else
                c.setMin_price(World.spotPKprice[Time.iTime]*0.8);
            return price;
        }
    } 
    
    @Override
    double calc_marge(Contract c) {
        double marge = MargeFuzzyCalculator.calc_marge(c);
        c.setMarge(marge);
        return marge;
    }
    @Override
    double calc_discount(Contract c) {
        double discount = DiscountMargeCalculator.calc_discount(c);
        return discount; 
    }

    @Override
    double set_discount_price(Contract c) {
        double discount = calc_discount(c);
        return c.price*(1 - discount*2);
    }

    @Override
    double set_discount_price(ContractSWUPK c_swupk) {
        
       double pk_discount = calc_discount(c_swupk);
       double new_pk_price = cost*(1 +c_swupk.pk_marge - pk_discount*2);
       //double new_pk_price = c_swupk.pk_price*(1-pk_discount);
       if (new_pk_price < c_swupk.min_pk_price){new_pk_price = c_swupk.min_pk_price;} 
       return new_pk_price;
    }
    
    
    
    @Override
    void deal_pk(Contract c) {
        change_capacity(c);
        money = money + c.price*c.volume;
        sales.get(c.year).add(Agreement.create_agreement(c));
        stats.recalc_deals_stat(c.year, c.product);
        stats.revenue[Time.iTime] = stats.revenue[Time.iTime] + c.price*c.volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.volume;
    }
    @Override
    void deal_swu_pk(ContractSWUPK c) {
        money = money + c.pk_price*c.pk_volume;
        change_capacity(c);
        Agreement[] aa = Agreement.create_agreement(c);
        sales.get(c.year).add(aa[0]);
        stats.recalc_deals_stat(c.year, aa[0].product);
        stats.revenue[Time.iTime] = stats.revenue[Time.iTime] + c.pk_price*c.pk_volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.pk_volume;
    }
    
    private void change_capacity(Contract c){
        switch (ScenarioTest.mining_type){
            case ScenarioTest.IAEA_MINING: change_iaea_capacity(c);break;
            case ScenarioTest.TENEX_MINING: change_tenex_capacity(c);break;
        }
    }
    private void change_capacity(ContractSWUPK c){
        switch (ScenarioTest.mining_type){
            case ScenarioTest.IAEA_MINING: change_iaea_capacity(c);break;
            case ScenarioTest.TENEX_MINING: change_tenex_capacity(c);break;
        }
    }
    private void change_tenex_capacity(ContractSWUPK c){
        if (deposits.size() > 0){
            ArrayList<DepositTenex> prod_deposits = choose_prod_deposit(c, c.pk_volume);
            double remains = c.pk_volume;
            for (DepositTenex prod_deposit : prod_deposits){
                double A = Math.min(remains, prod_deposit.capacity);
                stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + prod_deposit.cost*A;
                prod_deposit.extract(A);
                remains  = remains - A;
            }
            capacity = capacity - c.pk_volume;
        }else{
            capacity = capacity - c.pk_volume;
        }
    }
    
    private void change_tenex_capacity(Contract c){
        if (deposits.size() > 0){
            ArrayList<DepositTenex> prod_deposits = choose_prod_deposit(c, c.volume);
            double remains = c.volume;
            for (DepositTenex prod_deposit : prod_deposits){
                double A = Math.min(remains, prod_deposit.capacity);
                stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + prod_deposit.cost*A;
                prod_deposit.extract(A);
                remains  = remains - A;
            }
            capacity = capacity - c.volume;
        }else{
            capacity = capacity - c.volume;
        }
    }
    
    private void change_iaea_capacity(Contract c){
        if (pseudocountries.size() > 0){
            ArrayList<PseudoCountry> pseudoList = choose_delivery_country(c,0);
            double remains = c.volume;
            for (PseudoCountry pseudo : pseudoList) {
                double A = Math.min(remains, pseudo.capacity);
                stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + pseudo.current_cost*A;
                pseudo.extract(A);
                remains  = remains - A;
            }
            
            capacity = capacity - c.volume;
        }else{
            capacity = capacity - c.volume;
        }
    }
    void decrease_capacity(Contract c){
        int n = DMath.Arithmetic.indexOf(Time.years, c.year);
        all_capacity[n] = all_capacity[n] - c.volume;
    }
    private void change_iaea_capacity(ContractSWUPK c){
        if (pseudocountries.size() > 0){
            ArrayList<PseudoCountry> pseudoList = choose_delivery_country(c);
            double remains = c.pk_volume;
            for (PseudoCountry pseudo : pseudoList) {
                double A = Math.min(remains, pseudo.capacity);
                stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + pseudo.current_cost*A;
                pseudo.extract(A);
                remains  = remains - A;
            }
            capacity = capacity - c.pk_volume;
        }else{
            capacity = capacity - c.pk_volume;
        }
    }
    
    private double calc_cost(){
        switch (ScenarioTest.mining_type){
            case ScenarioTest.IAEA_MINING: return calc_iaea_cost();
            case ScenarioTest.TENEX_MINING: return calc_tenex_cost();
            default: return calc_iaea_cost();
        }
        
    }
    
    private double calc_tenex_cost(){
        if (deposits.size() > 0){
        ArrayList<DepositTenex> My_deposits = DepositTenex.reasemble_deposits_to_tenex_format(deposits);
        ArrayList<DepositTenex> not_exosted_deposits = new ArrayList<>();
        My_deposits.stream().filter(a -> a.capacity>0).allMatch(a -> not_exosted_deposits.add(a));
        if (not_exosted_deposits.isEmpty()) return Double.POSITIVE_INFINITY;
        double[] cp = new double[not_exosted_deposits.size()];
        int i = 0;
        for (DepositTenex not_exosted_deposit : not_exosted_deposits) {
            cp[i] = not_exosted_deposit.cost;
            i++;
        }
        int cp_i = DMath.Arithmetic.max(cp);
        cost = cp[cp_i];
        return cp[cp_i];
        }else{
            return cost;
        }
    }
    

    
    private double calc_iaea_cost(){
        if (pseudocountries.size() > 0){
            PseudoCountry pseudo = choose_cost_country(); 
            cost = pseudo.current_cost;
            return pseudo.current_cost;
        }else{
            return cost;
        }
    }
    
    private PseudoCountry choose_cost_country(){
        // нужно выбрать страну c максимальной себестоимостью
        int i = 0;
        double[] costs = new double[pseudocountries.size()];
        
        for (PseudoCountry pseudocountry : pseudocountries) {
            costs[i] = pseudocountry.current_cost;
            if (pseudocountry.capacity == 0)
                costs[i] = Double.NEGATIVE_INFINITY;
            i++;
        }
        int[] inds = DMath.Arithmetic.sort_B(costs);
        double curr_spot;
        if (Time.iTime == 0)
            curr_spot = World.start_PK_price;
        else
            curr_spot = World.spotPKprice[Time.iTime];
        for (int ind : inds) {
            
            if (costs[ind] < curr_spot*1.4 && costs[ind] > 0){
                PseudoCountry my_pseudo = pseudocountries.get(ind);
                return my_pseudo;
            }
        }
        int[] inds2 = DMath.Arithmetic.sort_A(costs);
        for (int j : inds2) {
            if (costs[j] > 0){
                PseudoCountry my_pseudo = pseudocountries.get(j);
                return my_pseudo;}
        }
        int n = DMath.Arithmetic.min(costs);
        
        PseudoCountry my_pseudo = pseudocountries.get(n);
        return my_pseudo;
    }
    
    private ArrayList choose_delivery_country(ContractSWUPK c){
        int i = 0;
        double[] costs = new double[pseudocountries.size()];
        
        for (PseudoCountry pseudocountry : pseudocountries) {
            costs[i] = pseudocountry.current_cost + getAgent().getLogistics(Contract.P_PK, c.get_buyer().getAgent().getName(), c.year);
            if (pseudocountry.capacity <= c.pk_volume)
                costs[i] = Double.POSITIVE_INFINITY;
            i++;
        }
        int[] indexes = DMath.Arithmetic.sort_A(costs);
        int n = 0;
        double remains = c.pk_volume; 
        ArrayList<PseudoCountry> pseudoList = new ArrayList<>();
        while (remains > 0){
            try{
                PseudoCountry my_pseudo = pseudocountries.get(indexes[n]);
                remains = remains - my_pseudo.capacity;
                pseudoList.add(my_pseudo);
                n++;  
            }  catch(java.lang.ArrayIndexOutOfBoundsException e)
            {
                System.err.println("dd");
            }
        }

        return pseudoList;
    }
    
    private ArrayList choose_delivery_country(Contract c, int A){
        // нужно выбрать страну c максимальной себестоимостью
        int i = 0;
        double[] costs = new double[pseudocountries.size()];
        
        for (PseudoCountry pseudocountry : pseudocountries) {
            costs[i] = pseudocountry.current_cost + getAgent().getLogistics(Contract.P_PK, c.get_buyer().getAgent().getName(), c.year);
            if (pseudocountry.capacity <= c.volume)
                costs[i] = Double.POSITIVE_INFINITY;
            i++;
        }
        int[] indexes = DMath.Arithmetic.sort_A(costs);
        int n = 0;
        double remains = c.volume; 
        ArrayList<PseudoCountry> pseudoList = new ArrayList<>();
        while (remains > 0){
            if (n >= indexes.length ){
                System.out.println("EROOORRR");  
            }
                PseudoCountry my_pseudo = pseudocountries.get(indexes[n]);
            remains = remains - my_pseudo.capacity;
            pseudoList.add(my_pseudo);
            n++;    
        }

        return pseudoList;
    }
    
    private  ArrayList<DepositTenex> choose_prod_deposit(Contract c, double Volume){
        int i = 0;
        ArrayList<DepositTenex> Tdepoits = DepositTenex.reasemble_deposits_to_tenex_format(deposits);
        double[] costs = new double[Tdepoits.size()];
        for (DepositTenex Tdepoit : Tdepoits) {
            costs[i] = Tdepoit.cost + getAgent().getLogistics(Contract.P_PK, c.get_buyer().getAgent().getName(), c.year);
            if (Tdepoit.capacity <= Volume)
                costs[i] = Double.POSITIVE_INFINITY;
            i++;
        }
        int[] indexes = DMath.Arithmetic.sort_A(costs);
        int n = 0;
        double remains = Volume; 
        ArrayList<DepositTenex> TDList = new ArrayList<>();
        while (remains > 0){
            try{
                DepositTenex my_tdeposit = Tdepoits.get(indexes[n]);
                remains = remains - my_tdeposit.capacity;
                TDList.add(my_tdeposit);
                n++;  
            }  catch(java.lang.ArrayIndexOutOfBoundsException e)
            {
                System.err.println("dd");
            }
        }

        return TDList;
    }
    
    @Override
    void restore() {
        if(Time.iTime < (Time.lastTime-1)){
            capacity = 0;
            check_deposits();
            calc_cost();
            if (ScenarioTest.mining_type == ScenarioTest.IAEA_MINING){
                pseudocountries.forEach((pseudocountry) -> {
                    capacity = capacity + pseudocountry.capacity;
                });
            }else{
                this.deposits.forEach(deposit -> {
                    capacity = capacity + deposit.capacity;
                });
            }         
            all_capacity[Time.iTime +1] =  capacity;
            all_cost[Time.iTime] = cost;
            double reserve = 0;
            for (Deposit deposit : deposits) {
                reserve = reserve + deposit.volume;
            }
            all_reserves[Time.iTime] = reserve;
            check_pbase();
        }
        else{
            all_cost[Time.iTime] = cost;
        }
    }
    private void check_pbase(){
        if (World.Switch_PBase){
            double Spot;
            if (Time.iTime > 2){
                Spot = World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)]*0.9/(0.9+0.8+0.7) + 
                       World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime)]*0.8/(0.9+0.8+0.7) +
                       World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.7/(0.9+0.8+0.7);
            }
            else
                Spot = World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)];
            
            if (Spot*0.95 < cost*1.05){
                pbase = Producer.PBASE_COST;
            } else{
                pbase = Producer.PBASE_PRICE;
            }
        }
    }
    
    private void check_start_pbase(){
        if (World.Switch_PBase){
            double Spot = World.start_PK_price;
            if (Spot*0.8 < cost*1.1){
                pbase = Producer.PBASE_COST;
            } else{
                pbase = Producer.PBASE_PRICE;
            }
        }
    } 
    
    void check_deposits(){
        switch (ScenarioTest.mining_type){
            case ScenarioTest.IAEA_MINING:{
                deposits.forEach(deposit -> deposit.restore());
                update_pseudocountries();
                deposits.stream().filter((deposit) -> (deposit.on())).forEachOrdered((deposit) -> {
                    increase_pseudocountry(deposit);
                });
            }break;
            case ScenarioTest.TENEX_MINING:{
                deposits.forEach(deposit -> deposit.restore());
                update_pseudocountries();
                deposits.stream().filter((deposit) -> (deposit.on())).forEachOrdered((deposit) -> {
                    increase_pseudocountry(deposit);
                });
            }break;
        }
    }
    
    void increase_pseudocountry(Deposit deposit){
        PseudoCountry my = pseudocountries.stream().filter(pse -> pse.country.equals(deposit.country)).findFirst().get();
        my.increase(deposit.volume, deposit.capacity);
    }

    int getActive_year() {
        return (int) Double.NEGATIVE_INFINITY;
    }

    private void update_pseudocountries() {
        for (PseudoCountry pseudocountry : pseudocountries) {
            ArrayList<Deposit> deposit_list = new ArrayList();
            this.deposits.forEach(d -> {
                if(d.country.equals(pseudocountry.country))
                    deposit_list.add(d);
                }
            );
            double Local_capacity = 0;
            for (Deposit deposit : deposit_list) {
                Local_capacity  = deposit.capacity + Local_capacity;
            }
            pseudocountry.capacity = Local_capacity;
        }
    }

    void upgrade() {
        deposits.forEach(d -> d.upgrade());
        restore();
    }
    
    
}


class PseudoCountry{


    Country country; 
    double volume;
    double Svolume;
    double capacity;
    
    double current_cost;
    private static final Logger LOG = Logger.getLogger(PseudoCountry.class.getName());
    
    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    public PseudoCountry(Country country, double Svolume, double capacity) {
        this.country = country;
        this.Svolume = Svolume;
        this.volume = Svolume;
        this.capacity = capacity;
        
        this.current_cost =  country.current_cost;
    }
    
    public void addListener(PropertyChangeListener listener) {
            support.addPropertyChangeListener(listener);
    }
 
    public void removeListener(PropertyChangeListener listener) {
            support.removePropertyChangeListener(listener);
    }
    
    static ArrayList<PseudoCountry> form(ArrayList<Deposit> deposits) {
        
        ArrayList<PseudoCountry> pseudocountries = new ArrayList<>();
        
        HashSet<Country> country_list = new HashSet();
        deposits.forEach((deposit) -> {
            country_list.add(deposit.country);
        });
        
        for (Country country : country_list) {
            
            Stream<Deposit> d = deposits.stream().filter( deposit ->  deposit.country.equals(country));
            Object[] dArr = d.toArray();
            ArrayList<Deposit> dArrDepostit = new ArrayList();
            for (Object a : dArr) {
                dArrDepostit.add((Deposit) a);
            }    
            dArr = null;
            double volume = 0; 
            double capacity = 0;
            for (Deposit a : dArrDepostit) {
                if (a.status[Time.iTime] == 1)
                {
                    volume = volume + a.volume;
                    capacity = capacity + a.capacity;
                }
            }
            
            
            
            PseudoCountry pseudocountry = new PseudoCountry(country, volume, capacity);
            pseudocountries.add(pseudocountry);
            
        }
        
        return pseudocountries; 
        
    }

    void extract(double volume) {
        
        this.volume = this.volume - volume;
        this.capacity = this.capacity - volume;
        if (capacity < 0)
            System.err.println("Capacity <0");
        this.country.new_cost(volume);
        current_cost = country.current_cost;
    }

    void increase(double volume) {
        this.volume = this.volume + volume;
    }
    
    void increase(double volume, double capacity ) {
        this.volume = this.volume + volume;
    }
}