/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import DMath.FuzzyRaiting;
import com.fuzzylite.Engine;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map.Entry;
import java.util.Optional;
import java.util.TreeMap;
import java.util.stream.Stream;


/**
 *
 * @author Dmitry
 */
public class Operator extends Actor{

    static double calc_clever_quotas(ArrayList<Operator> operators, double quota, int year) {
        if (quota == 0)
            return 0;
        if (quota == Double.POSITIVE_INFINITY)
            return Double.POSITIVE_INFINITY;
        ArrayList<Double> needsList = new ArrayList<>();
        for (Operator operator : operators) {
            for (Reactor reactor : operator.reactors) {
                needsList.add(reactor.needs[DMath.Arithmetic.indexOf(Time.years, year)]);
            }
        }
        Double[] needs = new Double[needsList.size()];
        needs = needsList.toArray(needs);
        double sum_needs = DMath.Arithmetic.sum(needs);
        double max_need = needs[DMath.Arithmetic.max(needs)];
        if (sum_needs == 0)
            return 0;    
        return Math.max(sum_needs*quota, (max_need+1));
    }

    static double calc_clever_swu_quotas(ArrayList<Operator> operators, double quota, int year) {
        if (quota == 0)
            return 0;
        if (quota == Double.POSITIVE_INFINITY)
            return Double.POSITIVE_INFINITY;
        ArrayList<Double> needsList = new ArrayList<>();
        for (Operator operator : operators) {
            for (Reactor reactor : operator.reactors) {
                needsList.add(Enrichment.calc_swu(reactor.needs[DMath.Arithmetic.indexOf(Time.years, year)],reactor.enrichment));
            }
        }
        Double[] needs = new Double[needsList.size()];
        needs = needsList.toArray(needs);
        double sum_needs = DMath.Arithmetic.sum(needs);
        double max_need = needs[DMath.Arithmetic.max(needs)];
        if (sum_needs == 0)
            return 0;    
        return Math.max(sum_needs*quota, (max_need+1));
    }
    
    static double calc_clever_pk_quotas(ArrayList<Operator> operators, double quota, int year) {
        if (quota == 0)
            return 0;
        if (quota == Double.POSITIVE_INFINITY)
            return Double.POSITIVE_INFINITY;
        ArrayList<Double> needsList = new ArrayList<>();
        for (Operator operator : operators) {
            for (Reactor reactor : operator.reactors) {
                needsList.add(Enrichment.calc_pk_volume(reactor.needs[DMath.Arithmetic.indexOf(Time.years, year)],reactor.enrichment));
            }
        }
        Double[] needs = new Double[needsList.size()];
        needs = needsList.toArray(needs);
        double sum_needs = DMath.Arithmetic.sum(needs);
        double max_need = needs[DMath.Arithmetic.max(needs)];
        if (sum_needs == 0)
            return 0;    
        return Math.max(sum_needs*quota, (max_need+1));
    }

    static void check_remix(ArrayList<Operator> Operators) {
        if(World.SWITCH_REMIX_VVER && Time.yTime > 2026 && Time.iTime < Time.years.length-1){
            ArrayList<Reactor> Ru_reactors = new ArrayList<>();
            HashMap<Operator, ArrayList> Ru_reactorsM = new HashMap();
            ArrayList<Operator> needed_ops = new ArrayList<>();
            for (Operator operator : Operators) {
                if (operator.id != 168){
                    ArrayList<Reactor> Ru_reactors0 = new ArrayList<>();
                    operator.reactors.stream().filter(z -> z.ru_design && z.type == Contract.EUP && (z.Rclass.equals("VVER-1000") || z.Rclass.equals("VVER-1200"))).forEachOrdered(z -> Ru_reactors0.add(z));
                    if (Ru_reactors0.size() >0){
                        needed_ops.add(operator);
                        Ru_reactorsM.put(operator, Ru_reactors0);
                        Ru_reactors.addAll(Ru_reactors0);
                    }
                }
            }
            double[] costs = new double[Ru_reactors.size()];
            int i = 0;
            for(Operator operator : needed_ops){
                ArrayList<Reactor> reacs = Ru_reactorsM.get(operator);
                for (Reactor reac : reacs) {
                    costs[i] = operator.calc_my_EUP_cost(reac);
                    i++;
                }
            }
            int I = DMath.Arithmetic.max(costs);
            Ru_reactors.get(I).set_trade_schema(Contract.REMIX);
        }
    }
    
    
    double[] supply;
    int max_iter=3;
    int max_round;
    double sum_needs;
    static double BACKEND_COST = 1000;
    static double FAB_COST = 350;
    
    EupRaitingFuzzyCalculator EupFuzzyCalculator;
    SwuRaitingFuzzyCalculator SWUFuzzyCalculator;
    MoxRaitingFuzzyCalculator MOXFuzzyCalculator;
    boolean has_pieces = false;
    SpentFuelStorage storage;
            
    public Operator() {
    }

    public Operator(ArrayList reactors, double[] supply, int max_iter, int max_round) {
        super();
        this.reactors = reactors;
        this.supply = supply;
        this.max_iter = max_iter;
        this.max_round = max_round;
        
        EupFuzzyCalculator = new EupRaitingFuzzyCalculator(this);
        SWUFuzzyCalculator = new SwuRaitingFuzzyCalculator(this);
        MOXFuzzyCalculator = new MoxRaitingFuzzyCalculator(this);
        
        storage = new SpentFuelStorage();
    }
    
    
    private void form_contracts(ArrayList<Reactor> Areactors) {
        int R = Areactors.size();
        double[] needs = new double[R];
        int i = 0;
        for (Reactor r: Areactors){
            needs[i] = Areactors.get(i).fuel_needs;
            i++;
        }
        sum_needs = DMath.Arithmetic.sum(needs);
        if (sum_needs > 0){
        // Есть нужды. на каждую ненулевую нужду нужно сделать контракт
            this.has_demand = true;
            int n = 0;
            for (double need: needs){
                if (need >0){
                   create_contracts(need, Areactors.get(n)); 
                }     
                n++;
            }
        }
        else{
            // Нет нужд. Торговли не будет
            this.has_demand = false;
        }
    }    
    
    @Override
    public void form_demand(){
        ArrayList<Reactor> all_reators = reactors;
        this.form_contracts(all_reators);
    }

    @Override
    public void form_ru_demand() {
        ArrayList<Reactor> Ru_reactors = new ArrayList<>();
        reactors.stream().filter(z -> z.ru_design).forEachOrdered(z -> Ru_reactors.add(z));
        this.form_contracts(Ru_reactors);
    }
    
    @Override
    public void form_remix_demand(){
        ArrayList<Reactor> Remix_reactors = new ArrayList<>();
        reactors.stream().filter(z -> z.type == Contract.REMIX).forEachOrdered(z -> Remix_reactors.add(z));
        this.form_contracts(Remix_reactors);
    }
    
    public void form_crumbs_demand(){
        Object[] unsatisfied_reactors_arr = this.reactors.stream().filter(reac -> reac.fuel_needs >0.00000001).toArray();
        double min_vol = 10;
        for (Object object : unsatisfied_reactors_arr) {
            Reactor r = (Reactor) object;
            has_pieces = true;
            int I = (int) Math.floor(r.fuel_needs/5);
            for (int i = 0; i < I; i++) {
                create_contracts(min_vol, r); 
            }
            if(r.fuel_needs%min_vol > 0)
                create_contracts(r.fuel_needs%min_vol, r); 
        }
        has_pieces = false;
    }
    @Override
    void restore() {
        for (Reactor reactor:reactors){
            reactor.restore_need();
            if (World.Switch_reac_deal_type){
                if(Time.iTime < Time.years.length-1 && Time.iTime > 2)
                check_switch(reactor);
                
            }
        }
    }
    
    private void check_switch(Reactor reactor){
//        if (Time.yTime == 2021 && id == 168){
//            System.err.println("TTT");
//        }
        if(reactor.ru_design)
        {
            switch (reactor.type) {
                case Contract.REMIX:
                    reactor.type = Contract.REMIX;
                    break;
                case Contract.MOX:
                    reactor.type = Contract.MOX;
                    break;
                default:
                    reactor.type = Contract.EUP;
                    break;
            }
            return;
        }
        if((this.getRegion().id == 5 || this.getRegion().id == 7) && Time.yTime >= 2020 && reactor.Rtype.equals("PWR")&& World.SWITCH_REMIX){
            double need = reactor.getNeeds(Time.yTime+1);
            double SWU_cost = calc_SWU_complex_cost(reactor) + BACKEND_COST*need + FAB_COST*need;
            double EUP_Cost = calc_EUP_cost(reactor) + BACKEND_COST*need + FAB_COST*need;
            double REMIX_Cost = calc_remix_cost(reactor);
            switch (reactor.type) {
                case Contract.EUP:
                    EUP_Cost = calc_my_EUP_cost(reactor) + BACKEND_COST*need + FAB_COST*need;
                    if (EUP_Cost > SWU_cost*1.3 && SWU_cost*1.3 < REMIX_Cost){
                        reactor.type = Contract.SWU_PK;
                    }
                    else if( EUP_Cost > REMIX_Cost*1.1  && REMIX_Cost*1.1 < SWU_cost ){
                        reactor.type = Contract.REMIX;
                    }
                    else
                        reactor.type = Contract.EUP;
                    break;
                case Contract.SWU_PK:
                    SWU_cost = calc_my_SWU_complex_cost(reactor) + BACKEND_COST*need + FAB_COST*need;
                    if (SWU_cost > EUP_Cost*1.3 &&  EUP_Cost*1.3 < REMIX_Cost){
                        reactor.type = Contract.EUP;
                    }
                    else if (SWU_cost > REMIX_Cost*1.1 && REMIX_Cost*1.1 < EUP_Cost){
                        reactor.type = Contract.REMIX;
                    }
                    else{
                        reactor.type = Contract.SWU_PK;
                    }   break;
                case Contract.REMIX:
                    REMIX_Cost = calc_my_REMIX_cost(reactor);
                    if (REMIX_Cost > EUP_Cost*1.3 &&  EUP_Cost*1.3 < SWU_cost){
                        reactor.type = Contract.EUP;
                    }
                    else if (REMIX_Cost > SWU_cost*1.3 && SWU_cost*1.3 < EUP_Cost){
                        reactor.type = Contract.SWU_PK;
                    }
                    else{
                        reactor.type = Contract.REMIX;
                    }   break;
                default:
                    break;
            }
            return;
        }
        if((this.getRegion().id == 7 || this.getRegion().id == 7) && Time.yTime == 2024 && reactor.Rtype.equals("PWR")&& false){
            reactor.type = Contract.PWR_MOX;
            return;
        }
        if (reactor.type == Contract.EUP){
            double SWU_cost = calc_SWU_complex_cost(reactor);
            double EUP_Cost = calc_my_EUP_cost(reactor);
            if (EUP_Cost > SWU_cost*1.2){
                reactor.type = Contract.SWU_PK;
            }
        }
        else if (reactor.type == Contract.SWU_PK){
            double SWU_cost = calc_my_SWU_complex_cost(reactor);
            double EUP_Cost = calc_EUP_cost(reactor);
            if (SWU_cost > EUP_Cost*1.2){
                reactor.type = Contract.EUP;
            }
        }
    }
    
    private void create_contracts(double need, Reactor r) {
        
        int type = define_type(r); 
        switch (type)
        {
            case Contract.SWU_PK:create_swu_contracts(need, r);break;
            case Contract.EUP:create_eup_contracts(need, r);break;
            case Contract.MOX:create_mox_contracts(need, r);break;
            case Contract.PWR_MOX:create_pwr_mox_contracts(need, r);break;
            case Contract.PK_Solo:create_pk_contracts(need, r);break;
            case Contract.REMIX:create_remix_contracts(need, r);break;
        }
    }

    void init_fuel_needs() {
        this.reactors.forEach((Reactor r) -> r.init_need() );
    }

    private int define_type(Reactor r) {
       // int type = (int) Math.round(Math.random()*3);
        int type = r.type;
        return type;
    }

    private void create_swu_contracts(double need, Reactor r) {
        TenderSWU.create_contracts(r, need, this);
    }

    private void create_eup_contracts(double need, Reactor r) {
        TenderEUPSolo.create_contracts(r, need, this);
    }

    private void create_mox_contracts(double need, Reactor r) {
        TenderMOX.create_mox_contracts(r, need, this);
    }
    private void create_pk_contracts(double need, Reactor r) {
        TenderPKSolo.create_contracts(r, need, this);
    }
    private void create_pwr_mox_contracts(double need, Reactor r) {
        TenderEUP.create_contracts(r, need/3*2, this);
        TenderMOX.create_mox_contracts(r, Reactor.recalc_mox(need/3), this);
    }
    private void create_remix_contracts(double need, Reactor r) {
        TenderRemix.create_contracts(r, need, this);
    }
    @Override
    void deal_mox(Contract c) {
        // Уменьшить потребность
        int id_r = reactors.indexOf(c.get_reactor());
        reactors.get(id_r).satisfy_needs(c.volume);
        sum_needs = sum_needs - c.volume;
        // Заплатить деньги
        this.money = this.money - c.price*c.volume;
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + c.price*c.volume;
    }
    @Override
    void deal_remix(Contract c) {
        // Уменьшить потребность
        int id_r = reactors.indexOf(c.get_reactor());
        reactors.get(id_r).satisfy_needs(c.volume);
        sum_needs = sum_needs - c.volume;
        // Заплатить деньги
        this.money = this.money - c.price*c.volume;
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + c.price*c.volume;
    }
    @Override
    void deal_eup(Contract c) {
        // Уменьшить потребность
        int id_r = reactors.indexOf(c.get_reactor());
        reactors.get(id_r).satisfy_needs(c.volume);
        sum_needs = sum_needs - c.volume;
        // Заплатить деньги
        this.money = this.money - c.price*c.volume;
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + c.price*c.volume;
        storage.storeWaste();
    }
    
    @Override
    void deal_swu_pk(ContractSWUPK c){
       int id_r = reactors.indexOf(c.get_reactor());
        reactors.get(id_r).satisfy_needs(c.volume);
        sum_needs = sum_needs - c.volume;
        // Заплатить деньги
        this.money = this.money - c.en_price*c.en_volume - c.pk_price*c.pk_volume; 
        Agreement[] aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa[0]);
        purchases.get(c.year).add(aa[1]);
        stats.recalc_deals_stat(c.year, aa[0].product);
        stats.recalc_deals_stat(c.year, aa[1].product);
        stats.write_detailed_vol_info(aa[0]);
        stats.write_detailed_vol_info(aa[1]);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + c.en_price*c.en_volume + c.pk_price*c.pk_volume;
        storage.storeWaste();
    }
    @Override
    void deal_pk(Contract c){
        int id_r = reactors.indexOf(c.get_reactor());
        reactors.get(id_r).satisfy_needs(c.volume);
        sum_needs = sum_needs - c.volume;
        this.money = this.money - c.price*c.volume;
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] + c.price*c.volume;
        storage.storeWaste();
    }
    
    @Override
    double calc_prioritet(Contract c) {
        switch( c.type){
            case Contract.EUP: return calc_eup_prioritet(c);
            case Contract.SWU_PK: return calc_swu_prioritet(c);
            case Contract.MOX: return calc_mox_prioritet(c);
            case Contract.PK_Solo: return calc_pk_prioritet(c);
            case Contract.REMIX: return calc_mox_prioritet(c);
            default: throw new Error("Unknown contract type"); 
        }
    }
    
    double calc_eup_prioritet(Contract c) {
        return EupFuzzyCalculator.calc_prioritet(c); //To change body of generated methods, choose Tools | Templates.
    }
    double calc_swu_prioritet(Contract c) {
        return SWUFuzzyCalculator.calc_prioritet((ContractSWUPK) c); //To change body of generated methods, choose Tools | Templates.
    }
    double calc_mox_prioritet(Contract c) {
        return MOXFuzzyCalculator.calc_prioritet(c); //To change body of generated methods, choose Tools | Templates.
    }
    double calc_pk_prioritet(Contract c) {
        return SWUFuzzyCalculator.calc_pk_prioritet(c); //To change body of generated methods, choose Tools | Templates.
    }
    @Override
    boolean is_acceptаble(Contract W) {
        
        boolean failed_quota = check_quota(W);

        if (failed_quota){
            // Контракт должен отменяться
            W.cancel();
            W.reason = Contract.REASON_QUOTA;
            return false;
        }
        double p = W.historic_prioritet.lastElement();
        if ((W.historic_prioritet.size()) > max_iter) {
            return true;
        }
        return super.is_acceptаble(p); //To change body of generated methods, choose Tools | Templates.
    }
    
    public void addReactor(Reactor newReactor){
        this.reactors.add(newReactor);
    }

    @Override
    double get_sum_neeeds() {
        return sum_needs;
    }

    private double calc_SWU_complex_cost(Reactor reactor) {
        double need = reactor.getNeeds(Time.yTime+1);
        double SWU_need = Enrichment.calc_swu(need, reactor.enrichment);
        double SWU_price;
        if (Time.iTime > 4){
             SWU_price = World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)]*0.9/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime)]*0.5/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.3/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.2/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.1/(0.9+0.5+0.3+0.2+0.1);
                    }
        else
            SWU_price = World.spotSWUprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)];
        double SWU_cost = SWU_need*SWU_price;
        
        double PK_price;
        if (Time.iTime > 4){
            PK_price = World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)]*0.9/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime)]*0.5/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.3/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.2/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.1/(0.9+0.5+0.3+0.2+0.1);
                    }
        else
            PK_price = World.spotPKprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)];
        
        double PK_need = Enrichment.calc_pk_volume(need, reactor.enrichment);
        double PK_cost = PK_need*PK_price;
        
        return SWU_cost + PK_cost;
    }
    
    private double calc_my_SWU_complex_cost(Reactor reactor) {
        double need = reactor.getNeeds(Time.yTime+1);
        double SWU_need = Enrichment.calc_swu(need, reactor.enrichment);
        double en_price;
        double en_price1;
        Optional<Agreement> B = purchases.get(Time.yTime).stream().filter(a ->( a.product==Contract.P_SWU && a.contract.get_reactor().equals(reactor) )).findFirst();
        if (B.isPresent()){
                Agreement A = B.get();
                en_price1 = A.price;
                        }
            else
                en_price1 = stats.swu_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime)];
        if (Time.iTime > 2){
            double en_price2;
            Optional<Agreement> B2 = purchases.get(Time.yTime-1).stream().filter(a ->( a.product==Contract.P_SWU && a.contract.get_reactor().equals(reactor) )).findFirst();
            if (B2.isPresent()){
                Agreement A = B2.get();
                en_price2 = A.price;
                        }
            else
                en_price2 = stats.swu_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)];
            
            double en_price3;
            Optional<Agreement> B3 = purchases.get(Time.yTime-2).stream().filter(a ->( a.product==Contract.P_SWU && a.contract.get_reactor().equals(reactor) )).findFirst();
            if (B3.isPresent()){
                Agreement A = B3.get();
                en_price3 = A.price;
                        }
            else
                en_price3 = stats.swu_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-2)];
            
            en_price = en_price1*0.9/(0.9+0.8+0.7) +
                    en_price2*0.8/(0.9+0.8+0.7) +
                    en_price3*0.9/(0.9+0.8+0.7);
            
        }
        else
        {en_price = en_price1;}
        
        double SWU_cost = SWU_need*en_price;
        
        double PK_need = Enrichment.calc_pk_volume(need, reactor.enrichment);
        double pk_price;
        double pk_price1;
        
        Optional<Agreement> P = purchases.get(Time.yTime).stream().filter(a ->( a.product==Contract.P_PK && a.contract.get_reactor().equals(reactor) )).findFirst();
        if (P.isPresent()){
                Agreement A = P.get();
                pk_price1 = A.price;
                        }
            else
                pk_price1 = stats.pk_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime)];
        if (Time.iTime > 2){
            double pk_price2;
            Optional<Agreement> P2 = purchases.get(Time.yTime-1).stream().filter(a ->( a.product==Contract.P_PK && a.contract.get_reactor().equals(reactor) )).findFirst();
            if (P2.isPresent()){
                    Agreement A = P2.get();
                    pk_price2 = A.price;
                        }
            else
                pk_price2 = stats.pk_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)];
            
            double pk_price3;
            Optional<Agreement> P3 = purchases.get(Time.yTime-2).stream().filter(a ->( a.product==Contract.P_PK && a.contract.get_reactor().equals(reactor) )).findFirst();
            if (P3.isPresent()){
                    Agreement A = P3.get();
                    pk_price3 = A.price;
                }
            else
                pk_price3 = stats.pk_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-2)];
            
            pk_price = en_price1*0.9/(0.9+0.8+0.7) +
                    pk_price2*0.8/(0.9+0.8+0.7) +
                    pk_price3*0.9/(0.9+0.8+0.7);
            }
        else{
            pk_price = pk_price1;
        }
        double PK_cost = PK_need*pk_price;
        
        return SWU_cost + PK_cost;
    }
    
    private double calc_EUP_cost(Reactor reactor){
        double need = reactor.getNeeds(Time.yTime+1);
        double EUP_price;
        if (Time.iTime > 4){
            EUP_price = World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)]*0.9/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime)]*0.5/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.3/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.2/(0.9+0.5+0.3+0.2+0.1) +
                    World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)]*0.1/(0.9+0.5+0.3+0.2+0.1);
                    }
        else
            EUP_price = World.spotEUPprice[DMath.Arithmetic.indexOf(Time.years, Time.yTime+1)];
        return need*EUP_price;
    }

    private double calc_my_EUP_cost(Reactor reactor) {
        double need = reactor.getNeeds(Time.yTime+1);
        Agreement C;
        double price1;
        double price;
        try (Stream<Agreement> A = purchases.get(Time.yTime).stream().filter(a -> a.contract.get_reactor().equals(reactor))) {
            Optional<Agreement> B = A.findFirst();
            if (B.isPresent()){
                C = B.get();
                price1 = C.price;
                        }
            else
                price1 = stats.eup_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime)];
        }
        if (Time.iTime > 2){
            double price2;
            try (Stream<Agreement> A = purchases.get(Time.yTime-1).stream().filter(a -> a.contract.get_reactor().equals(reactor))) {
                Optional<Agreement> B = A.findFirst();
                if (B.isPresent()){
                    C = B.get();
                    price2 = C.price;
                            }
                else
                    price2 = stats.eup_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-1)];
            }
            
            double price3;
            try (Stream<Agreement> A = purchases.get(Time.yTime-2).stream().filter(a -> a.contract.get_reactor().equals(reactor))) {
                Optional<Agreement> B = A.findFirst();
                if (B.isPresent()){
                    C = B.get();
                    price3 = C.price;
                            }
                else
                    price3 = stats.eup_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime-2)];
            }
            
            price = price1*0.9/(0.9+0.8+0.7) +
                    price2*0.8/(0.9+0.8+0.7) +
                    price3*0.9/(0.9+0.8+0.7);
            
        }
        else
            price = price1;
        
        return need*price;
    }

    private double calc_remix_cost(Reactor reactor) {
        double need = reactor.getNeeds(Time.yTime+1);
        double need_remix = Reactor.recalc_remix(need, reactor);
        
        return need_remix*World.spotREMIXprice[Time.iTime+1];
    }

    private double calc_my_REMIX_cost(Reactor reactor) {
        double need = reactor.getNeeds(Time.yTime+1);
        double need_remix = Reactor.recalc_remix(need, reactor);
        Agreement C;
        double price;
        try (Stream<Agreement> A = purchases.get(Time.yTime).stream().filter(a -> a.contract.get_reactor().equals(reactor))) {
            Optional<Agreement> B = A.findFirst();
            if (B.isPresent()){
                C = B.get();
                price = C.price;
                        }
            else
                price = stats.remix_price[DMath.Arithmetic.indexOf(Time.years, Time.yTime)];
        }
        return need_remix*price;
    }

    @Override
    void calc_needed_waste(double volume) {
        double puShare = 0.17;
        double uShare = 0.83;
        double neededPu = volume * puShare;
        double neededU = volume * uShare;
    }
    
    
    
    class SpentFuelStorage {
        public static final double mass = 21000;
        public static final double uMass = 19800;
        public static final double puMass = 213;
        // Добавить в реактор
        private ArrayList<TreeMap<String, Double>> list; 
        //Добавить еще 2 значения (минорные актиниды и продукты деления(отходы))
        
        public SpentFuelStorage() {
            list = new ArrayList<TreeMap<String, Double>>();
        }

        private void storeWaste() {
            TreeMap<String, Double> newWaste = new TreeMap<String, Double>();
            newWaste.put("uranium", uMass);
            newWaste.put("plutonium", puMass);
            newWaste.put("neptunium", 0.0);
            newWaste.put("americium", 0.0);
            newWaste.put("curium", 0.0);
            list.add(newWaste);
        }
        
        public void manageWaste() {
            storeWaste();
            calc_needed_waste(0.0); //Надо, чтобы метод возвращал хранилище из требуемых элементов
            for (TreeMap<String, Double> storage : list) {
                for (Entry entry : storage.entrySet()) {
                    Double elemMass = (Double) entry.getValue();
                    if (elemMass != 0) {
                        // Написать разницу между mass и needed
                        //if (/* Разница == 0 */) {
                            //break;
                        //}
                    }
                }
            }
        }
        
        
    }
    
    
}
