/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import java.util.LinkedHashMap;

/**
 *
 * @author Dmitry
 */
public class RemixSupplier extends Producer{
    static double XU_REG = 0.008; //концентрация урана-235 в урановом регенерате
    static double MU_PU = 0.011; //массовая доля изотопов плутония в ОЯТ
    static double X9 = 0.68;// доля делящихся нуклидов плутония (239+241) в плутонии ОЯТ
    static double SFF = 0.05; // доля продуктов деления в ОЯТ при выгорании около 50 МВт•сут/кгU
    int max_iter = 3;
            
    static double calc_PK(Contract remix_contract, double tail) {
        double all_volume = remix_contract.volume;
        double eup_volume = calc_eup_volume(all_volume, remix_contract.enrichment, tail);
        
        double pk_volume = Enrichment.calc_pk_volume(eup_volume, remix_contract.enrichment, tail);
        
        return pk_volume;
    }
    
    static double calc_eup_volume(double all_volume, double enrichment, double tail){
        double u_share = calc_u_share(enrichment, tail);
        double eup_volume = u_share*all_volume;
        return eup_volume;
    }
    
    static double calc_u_share (double enrichment, double tail){
        double pu_share = calc_pu_share(enrichment);
        double reg_share = calc_reg_share(pu_share, enrichment, tail);
        
        double u_share = (1 - reg_share - pu_share);
        return u_share;
    }

    private static double calc_pu_share(double enrichment) {
        double pu_share = MU_PU*(X9 - XU_REG)/(enrichment - XU_REG);
        return pu_share;
    }

    private static double calc_reg_share(double pu_share, double enrichment, double tail) {
        double reg_share = (1 - SFF - pu_share)*(XU_REG - tail)/(enrichment - tail);
        return reg_share;
    }
    Enrichment partner;
    MoxMargeFuzzyCalculator MargeFuzzyCalculator;
    MoxDiscountFuzzyCalculator DiscountFuzzyCalculator;
    PKRaitingFuzzyCalculator PKFuzzyCalculator;
    
    public RemixSupplier(double[] all_capacity, double[] all_cost) {
        this.capacity = all_capacity[0];
        this.cost = all_cost[0];
        this.all_capacity = all_capacity;
        this.all_cost = all_cost;
        MargeFuzzyCalculator = new MoxMargeFuzzyCalculator(this);
        DiscountFuzzyCalculator = new MoxDiscountFuzzyCalculator(this);
        PKFuzzyCalculator = new PKRaitingFuzzyCalculator(this);
        stats = new RemixStat(this);
    }

    double getTail() {
        return partner.getTail();
    }

    public void setPartner(Enrichment partner) {
        this.partner = partner;
    }

    @Override
    void deal(Contract c) {
        switch (c.product){
            case Contract.P_PK: deal_pk(c); break;
            case Contract.P_REMIX: deal_remix(c);break;
            case Contract.P_SWU: deal_swu(c);break;
        }
                
    }
    
    @Override
    void deal_remix(Contract c) {
        money = money + c.price*c.volume;
        capacity = capacity - c.volume;
        Agreement aa = Agreement.create_agreement(c);
        sales.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  calc_pu_cost(c);
        stats.revenue[Time.iTime] =  stats.revenue[Time.iTime] + c.price*c.volume;
        stats.loaded[Time.iTime] = stats.loaded[Time.iTime] + c.en_volume;
    }

    @Override
    void deal_pk(Contract c) {
        this.money = this.money - c.price*c.volume; 
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.price*c.volume;
        TenderRemix.wake_father(c);
    }
    
    void deal_swu(Contract c){
        this.money = this.money - c.price*c.volume; 
        Agreement aa = Agreement.create_agreement(c);
        purchases.get(c.year).add(aa);
        stats.recalc_deals_stat(c.year, c.product);
        stats.write_detailed_vol_info(aa);
        stats.expenses[Time.iTime] = stats.expenses[Time.iTime] +  c.price*c.volume;
    }
    
    @Override
    double calc_price(Contract con) {
        double Rcost = calc_cost(con);
        double marge = calc_marge(con);
        con.setMin_price(Rcost/con.volume*(1+min_marge));
        double price = Rcost/con.volume*(1+marge);
        return price;
    }

    private double calc_cost(Contract con) {
        double Fab_SNF_cost = cost*con.volume;
        double Reg_cost = calc_reg_cost(con);
        double U_cost = calc_u_cost(con);
        double Rcost = Fab_SNF_cost + Reg_cost + U_cost;
        return Rcost;
    }
    private double calc_pu_cost(Contract con) {
        double pu_volume = calc_pu_share(con.enrichment)*con.volume;
        double pu_cost = pu_volume*cost;
        return pu_cost;
    }

    private double calc_reg_cost(Contract con) {
        double reg_volume = calc_reg_share(calc_pu_share(con.enrichment),con.enrichment, partner.getTail())*con.volume;
        
        double swu_volume = Enrichment.calc_swu(reg_volume, con.get_reactor().enrichment, partner.getTail());
        
        Contract fake_reg_contract = deal_fake_contract(swu_volume, con);
        
        double reg_cost = swu_volume*fake_reg_contract.price;
        return reg_cost;
    }
    
    
    private double calc_u_cost(Contract con) {
        Contract pk_contract = con.getSon(0);
        double pk_cost = pk_contract.volume*pk_contract.price;
        
        double u_volume = calc_eup_volume(con.volume, con.get_reactor().enrichment, partner.getTail());
        double swu_volume = Enrichment.calc_swu(u_volume, con.get_reactor().enrichment, partner.getTail());
        
        Contract fake_u_contract = deal_fake_contract(swu_volume, con);
        
        double u_cost = pk_cost + swu_volume*fake_u_contract.price;
        return u_cost;
    }
    
    private Contract deal_fake_contract(double swu_volume, Contract remix_con){
            Contract new_contract = new Contract();
            new_contract.level = Contract.LEVEL_H;
            new_contract.type = Contract.REMIX;
            new_contract.volume = swu_volume;
            new_contract.product = Contract.P_SWU;
            new_contract.enrichment = remix_con.enrichment;
            new_contract.historic_prioritet.add(10.0);
            new_contract.set_buyer(this);
            new_contract.set_seller(partner);
            new_contract.set_current_owner(this);
            new_contract.set_reactor(remix_con.get_reactor());
            new_contract.set_father(remix_con);
            new_contract.year = Time.yTime;
            
            new_contract.set_price(partner.get_min_price());
            new_contract.historic_price.add(new_contract.price);
            ArchiveChain.add(new_contract);
            
            partner.deal_swu(new_contract);
            this.deal(new_contract);
            return new_contract;
            
    }

    @Override
    double calc_prioritet(Contract c) {
         return PKFuzzyCalculator.calc_prioritet(c);
    }
    
    
    @Override
    double calc_marge(Contract c) {
       double marge = MargeFuzzyCalculator.calc_marge(c);
       return marge;
    }

    @Override
    double calc_discount(Contract c) {
        return DiscountFuzzyCalculator.calc_discount(c); 
    }
    
    @Override
    boolean is_acceptаble(Contract W) {
        double p = W.historic_prioritet.lastElement();
        if ((W.historic_prioritet.size()) > max_iter) {
            return true;
        }
        return super.is_acceptаble(p); //To change body of generated methods, choose Tools | Templates.
    }

    double calc_spot_price() {
        double need = 10;
        Contract con = new Contract();
        con.volume = need;
        con.enrichment = 0.045;
        double Fab_SNF_cost = cost*con.volume;
        double Reg_cost = calc_reg_false_cost(con);
        double U_cost = calc_u_false_cost(con);
        double Rcost = Fab_SNF_cost + Reg_cost + U_cost;
        return Rcost/need;
    }
    
    private double calc_u_false_cost(Contract con) {
        double u_volume = calc_eup_volume(con.volume, con.enrichment, partner.getTail());
        double pk_volume = Enrichment.calc_pk_volume(u_volume, con.enrichment, partner.getTail());
        double pk_cost = pk_volume*World.spotPKprice[Time.iTime+1];
        
        double swu_volume = Enrichment.calc_swu(u_volume, con.enrichment, partner.getTail());
        
        
        double u_cost = pk_cost + swu_volume*World.spotSWUprice[Time.iTime+1];
        return u_cost;
    }
    
    private double calc_reg_false_cost(Contract con) {
        double reg_volume = calc_reg_share(calc_pu_share(con.enrichment),con.enrichment, partner.getTail())*con.volume;
        
        double swu_volume = Enrichment.calc_swu(reg_volume, con.enrichment, partner.getTail());
        
        double reg_cost = swu_volume*partner.cost*(1+partner.min_marge);
        return reg_cost;
    }
}
class RemixStat extends Statistic{

    
    public RemixStat(Actor act) {
        super(act);
    }
    
    @Override
    public LinkedHashMap<String, Object> assembleActorSet(){
        LinkedHashMap<String, Object> Eimport = new LinkedHashMap<>(); 

        Eimport.put("year", Time.years);
        Eimport.put("Выручка", this.revenue);
        Eimport.put("Затраты", this.expenses);
        Eimport.put("Объем закупок ПК, т.", this.pk_volume);
        Eimport.put("Средняя цена покупки, &/кг", this.pk_price);
        Eimport.put("Аренда ЕРР, тыс. ЕРР", this.swu_volume);
        Eimport.put("Средняя ЕРР, &/ЕРР", this.swu_price);
        Eimport.put("Объем продажи РЕМИКС, т.", this.remix_volume);
        Eimport.put("Средняя цена продажи РЕМИКС, &/кг", this.remix_price);
        
        return Eimport;
    }
}