/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import DMath.Arithmetic;
import static DMath.Arithmetic.sum;
import DMath.FuzzyPkRaiting;
import DMath.FuzzyRaiting;
import com.fuzzylite.Engine;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.stream.Stream;

/**
 *
 * @author Dmitry
 */
class SwuRaitingFuzzyCalculator extends FuzzyCalculator{
    Operator operator;

    public SwuRaitingFuzzyCalculator(Operator operator) {
        this.operator = operator;
    }
    
    public double getSupplierRaiting(Contract c, Actor actor){
        Agent agent = operator.getAgent();
        return agent.getRelation(actor.getAgent().getName(),Time.yTime);
    }
    
    public double getRelSwuPrice(ContractSWUPK c){
        ArrayList<ContractSWUPK> cons = (ArrayList<ContractSWUPK>) c.getSBrothers().clone();
        cons.add(c);
        double[] prices = new double[cons.size()];
        int i = 0;
        for (ContractSWUPK con : cons) {
            prices[i] = con.en_price;
            i++;
        }
        int M = Arithmetic.max(prices);
        return c.en_price/prices[M];
    }
    
    public double getRelPkPrice(ContractSWUPK c){
        ArrayList<ContractSWUPK> cons = (ArrayList<ContractSWUPK>) c.getSBrothers().clone();
        cons.add(c);
        double[] prices = new double[cons.size()];
        int i = 0;
        for (ContractSWUPK con : cons) {
            prices[i] = con.pk_price;
            i++;
        }
        int M = Arithmetic.max(prices);
        return c.pk_price/prices[M];
    }
    
    public double getSupplierShare(Contract c, Actor actor){
        
        Double eup_value = operator.stats.eup.get(c.year).get(actor.getName());
        Double swu_value = operator.stats.swu.get(c.year).get(actor.getName());
        double needs = operator.stats.swu_needs[DMath.Arithmetic.indexOf(Time.years, c.year)];
        
        double eup2swu_value = Enrichment.calc_swu(eup_value, 0.04);
        
        return (swu_value + eup2swu_value)/needs;
        /*  try{
        HashMap v = ArchiveChain.getSpecificActorContract( c.year, operator);
        if (v == null){
        return 0;
        }
        Collection Values = v.values();
        ArrayList<Contract>ff = new ArrayList(Values);
        Stream<Contract> aa = ff.stream().filter(z -> z.get_seller().equals(c.get_seller()));
        Object[] tupoy_stream = aa.toArray();
        Contract[] cons = new Contract[tupoy_stream.length];
        int n = 0;
        for (Object object : tupoy_stream) {
        cons[n] = (Contract)object;
        n++;
        }
        if (cons.length >0){
        double[] sum_suuply = new double[cons.length];
        int i = 0;
        for (Contract con : cons) {
        sum_suuply[i] = con.get_reactor().fuel_needs;
        i++;
        }
        return sum(sum_suuply)/operator.sum_needs;
        }else{
        return 0;
        }
        }
        catch(java.lang.ClassCastException e){
        throw e;
        }
        catch(Exception e){
        throw e;
        }*/
    }
    
    public double getSpotPrice(ContractSWUPK c, short prod){
        switch (prod) {
            case Contract.P_SWU:
                return c.price/World.spotSWUprice[Time.iTime];
            case Contract.P_PK:
                return c.price/World.spotPKprice[Time.iTime];
            default:
                throw new Error("Not SWU_PK prod");
        }
    }
    
    public double getLogistic(Contract c, Actor actor){
        Agent my_agent = operator.getAgent();
        HashMap<String, Double> aa = my_agent.getLogistics(Contract.P_SWU, c.year);
        double T = Collections.max(aa.values());
        
        return aa.get(actor.getAgent().getName())/T*5;
    }
    
    public double getPKDemandSupply(Contract c){
        int ff = DMath.Arithmetic.indexOf(Time.years, c.year);
        double pk_demand = World.total_pk_demand[ff];
        double pk_supply = World.total_pk_supply[ff];
        
        return pk_demand/pk_supply;
    }
    
    public double getCost_price(ContractSWUPK c){
       double ans = (c.pk_price*c.pk_volume)/(c.pk_price*c.pk_volume+c.en_price*c.en_volume);
       return ans;
    }
    /*
    public double getExpectence(Contract c){
        return 5;
    }*/
    
    
    double calc_prioritet(ContractSWUPK c) {

        double en_raiting = calc_swu_prioritet(c);
        double pk_raiting = calc_pk_prioritet(c);
        return (en_raiting + pk_raiting*2)/3; 
    }
    
    double calc_swu_prioritet(ContractSWUPK c){
        Engine engine_swu = DMath.FuzzyStore.getEngine("contr", Contract.P_SWU,operator.getRegion().getBrain());
        FuzzyRaiting fis = new DMath.FuzzyRaiting(engine_swu);
        fis.setExpectense(getExpectence(c));
        fis.setLogistic(getLogistic(c, c.getEn_seller()));
        fis.setRelPrice(getRelSwuPrice(c));
        fis.setSpotPrice(getSpotPrice(c, Contract.P_SWU));
        fis.setSupplierRaiting(getSupplierRaiting(c, c.getEn_seller()));
        fis.setSupplierShare(getSupplierShare(c, c.getEn_seller()));
        
        
        double[] Inputs = fis.getInputValues();
        c.setRaiting_inputs(Inputs);
        
        double en_raiting = fis.eval();
        engine_swu.restart();
        return en_raiting;
    }
    
    double calc_pk_prioritet(ContractSWUPK c){
        Engine engine_pk = DMath.FuzzyStore.getEngine("contr", Contract.P_PK,operator.getRegion().getBrain());
        FuzzyPkRaiting pk_fis = new DMath.FuzzyPkRaiting(engine_pk);
        pk_fis.setRelPrice(getRelPkPrice(c));
        pk_fis.setSpotPrice(getSpotPrice(c, Contract.P_PK));
        pk_fis.setSupplierRaiting(getSupplierRaiting(c, c.getPk_seller()));
        pk_fis.setDemand_supply(getPKDemandSupply(c));
        pk_fis.setCost_price(getCost_price(c));
        double pk_raiting = pk_fis.eval();
        engine_pk.restart();
        return pk_raiting;
    }
    
    
    double calc_pk_prioritet(Contract c) {
        Engine engine_pk = DMath.FuzzyStore.getEngine("contr",Contract.P_PK, operator.getRegion().getBrain());
        FuzzyPkRaiting pk_fis = new DMath.FuzzyPkRaiting(engine_pk);
        pk_fis.setRelPrice(getRelPrice(c));
        pk_fis.setSpotPrice(getSpotPrice(c));
        pk_fis.setSupplierRaiting(getSupplierRaiting(c, c.get_seller()));
        pk_fis.setDemand_supply(getPKDemandSupply(c));
        pk_fis.setCost_price(getCost_price(c));
        double pk_raiting = pk_fis.eval();
        return pk_raiting;
    }


}
