/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package smmum_alpha;

import DMath.Arithmetic;
import static java.lang.Math.max;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;

/**
 *
 * @author Dima
 */
public class FuzzyCalculator {
    
    
    public double getRelPrice(Contract c){
        ArrayList<Contract> cons = (ArrayList<Contract>) c.getBrothers().clone();
        cons.add(c);
        double[] prices = new double[cons.size()];
        int i = 0;
        for (Contract con : cons) {
            prices[i] = con.getPrice();
            i++;
        }
        int M = Arithmetic.max(prices);
        return c.getPrice()/prices[M];
    }
    public double getSupplierRaiting(Contract c){
        Agent agent = c.get_buyer().getAgent();
        return agent.getRelation(c.get_seller().getAgent().getName(),Time.yTime);
    }
    public double getBuyerRaiting(Contract c){
        if (c.getClass().equals(ContractSWUPK.class)){
            return getBuyerRaiting((ContractSWUPK)c);
        }
        Agent agent = c.get_seller().getAgent();
        return agent.getRelation(c.get_buyer().getAgent().getName(),Time.yTime);
    }

    public double getBuyerRaiting(ContractSWUPK c){
        Agent agent = c.getEn_seller().getAgent();
        return agent.getRelation(c.get_buyer().getAgent().getName(),Time.yTime);
    }
    public double getDemandSupply(Contract c){
        int ff = DMath.Arithmetic.indexOf(Time.years, c.year);
        double demand;
        double supply;
        switch (c.product){
            case Contract.P_PK: {
                demand = World.total_pk_demand[ff];
                supply = World.total_pk_supply[ff] - World.tbulletin.trader.all_capacity[ff];
                break;
            }
            case Contract.P_OUP:{
                demand = World.total_swu_demand[ff];
                supply = World.total_swu_supply[ff];
                break;
            }
            case Contract.P_SWU:{
                demand = World.total_swu_demand[ff];
                supply = World.total_swu_supply[ff];
                break;
            }
            case Contract.P_MOX:{
                demand = World.total_mox_demand[ff];
                supply = World.total_mox_supply[ff];
                break;
            }
            case Contract.P_REMIX:{
                demand = World.total_remix_demand[ff];
                supply = World.total_remix_supply[ff];
                break;
            }
            default: {throw new Error("Wrong product");}
        }
        return demand/supply;
    }
    
    public double getSpotPrice(Contract c){
        switch (c.product){
            case Contract.P_PK: return c.price/World.spotPKprice[Time.iTime];
            case Contract.P_OUP: return c.price/World.spotEUPprice[Time.iTime];
            case Contract.P_SWU: return c.price/World.spotSWUprice[Time.iTime];
            case Contract.P_MOX: return c.price/World.spotMOXprice[Time.iTime];
            case Contract.P_REMIX: return c.price/World.spotREMIXprice[Time.iTime];
            default: {throw new Error("Wrong product");}
        }
    }
    
    public double getExpectence(Contract c){
        if (c.year == Time.years[0])
        return 0.0;
        else{
            double[] hist_price = new double[Math.min(Time.iTime+1,5)];
            switch (c.product){
                case Contract.P_MOX:{
                    hist_price = Arrays.copyOfRange(World.spotMOXprice,max(0, Time.iTime - 4),Time.iTime+1 );
                    break;
                }
                case Contract.P_PK:{
                    hist_price = Arrays.copyOfRange(World.spotPKprice,max(0, Time.iTime - 4),Time.iTime+1 );
                    break;
                }
                case Contract.P_OUP:{
                    hist_price = Arrays.copyOfRange(World.spotEUPprice,max(0, Time.iTime - 4),Time.iTime+1 );
                    break;
                }
                case Contract.P_SWU:{
                    hist_price = Arrays.copyOfRange(World.spotSWUprice,max(0, Time.iTime - 4),Time.iTime+1 );
                    break;
                }
                case Contract.P_REMIX:{
                    hist_price = Arrays.copyOfRange(World.spotREMIXprice,max(0, Time.iTime - 4),Time.iTime+1 );
                    break;
                }
                default: throw new Error("Unknown product");
            }
            double[] diff_price = new double[hist_price.length - 1];
            for (int i = 1; i < hist_price.length; i++) {
                diff_price[i-1] =  (hist_price[i]-hist_price[i-1])/hist_price[i-1];  
            }
            return DMath.Arithmetic.mean(diff_price)/(-0.5*(5-diff_price.length));
        }
    }
    
    public double getPriceDynamic(Contract Cons){
    if (Time.iTime == 0){
        return 0;
    }else{
            switch (Cons.product){
                case Contract.P_MOX: return (World.spotMOXprice[Time.iTime]-World.spotMOXprice[Time.iTime-1])/World.spotMOXprice[Time.iTime-1];
                case Contract.P_REMIX: return (World.spotREMIXprice[Time.iTime]-World.spotREMIXprice[Time.iTime-1])/World.spotREMIXprice[Time.iTime-1];
                case Contract.P_OUP: return (World.spotEUPprice[Time.iTime]-World.spotEUPprice[Time.iTime-1])/World.spotEUPprice[Time.iTime-1];
                case Contract.P_PK: return (World.spotPKprice[Time.iTime]-World.spotPKprice[Time.iTime-1])/World.spotPKprice[Time.iTime-1];
                case Contract.P_SWU: return (World.spotSWUprice[Time.iTime]-World.spotSWUprice[Time.iTime-1])/World.spotSWUprice[Time.iTime-1];
                default: throw new Error("Undefunied priduct");
            
            }
        }
    }
    
    public double getCrossPriceDynamic(Contract Cons){
    if (Time.iTime == 0){
        return 0;
    }else{
            switch (Cons.product){
                case Contract.P_MOX: return (World.spotPKprice[Time.iTime]-World.spotPKprice[Time.iTime-1])/World.spotPKprice[Time.iTime-1];
                case Contract.P_REMIX: return (World.spotPKprice[Time.iTime]-World.spotPKprice[Time.iTime-1])/World.spotPKprice[Time.iTime-1];
                case Contract.P_OUP: return (World.spotPKprice[Time.iTime]-World.spotPKprice[Time.iTime-1])/World.spotPKprice[Time.iTime-1];
                case Contract.P_PK: return (World.spotSWUprice[Time.iTime]-World.spotSWUprice[Time.iTime-1])/World.spotSWUprice[Time.iTime-1];
                case Contract.P_SWU: return (World.spotPKprice[Time.iTime]-World.spotPKprice[Time.iTime-1])/World.spotPKprice[Time.iTime-1];
                default: throw new Error("Undefunied priduct");
            
            }
        }
    }
    
    public double getCostPrice(Contract Cons){
        throw new UnsupportedOperationException("Реализованно в подклассе");
    }
    public double getCost_price(Contract c){
       double ans = (c.volume*c.price)/(World.spotEUPprice[Time.iTime]*c.volume*(World.natural_enrichment - World.waste_enrichment)/(0.045-World.waste_enrichment));
       return ans;
    }
    
    public double getLogistic(Contract c){
        Agent my_agent = c.get_buyer().getAgent();
        HashMap<String, Double> aa = my_agent.getLogistics(c.product, c.year);
        double T = Collections.max(aa.values());
        if (T == 0) return 2.5;
        return aa.get(c.get_seller().getAgent().getName())/T*5;
    }
    
    public double getCostRank(Contract Cons){
        
        switch (Cons.product){
            case Contract.P_MOX: {
                ArrayList<MoxSupplier> all_actors = World.tbulletin.my_mox_suppliers;
                double[] all_costs = all_actors.stream().mapToDouble(a -> a.cost).toArray();
                int[] all_ids = all_actors.stream().mapToInt(a -> a.id).toArray();
                int m = DMath.Arithmetic.max(all_costs);
                int my = DMath.Arithmetic.indexOf(all_ids, Cons.get_seller().id);
                return all_costs[my]/all_costs[m];
            }
            case Contract.P_OUP: {
                ArrayList<Enrichment> all_actors = World.tbulletin.my_enrichments;
                double[] all_costs = all_actors.stream().mapToDouble(a -> a.cost).toArray();
                int[] all_ids = all_actors.stream().mapToInt(a -> a.id).toArray();
                int m = DMath.Arithmetic.max(all_costs);
                int my = DMath.Arithmetic.indexOf(all_ids, Cons.get_seller().id);
                return all_costs[my]/all_costs[m];

            }
            case Contract.P_PK: {
                ArrayList<Mining> all_actors = World.tbulletin.my_minings;
                double[] all_costs = all_actors.stream().mapToDouble(a -> a.cost).toArray();
                int[] all_ids = all_actors.stream().mapToInt(a -> a.id).toArray();
                int m = DMath.Arithmetic.max(all_costs);
                int my = DMath.Arithmetic.indexOf(all_ids, Cons.get_seller().id);
                return all_costs[my]/all_costs[m];
            }
            default: throw new Error("Undefunied priduct");

        }  
    }
    
     public double getCostRank(ContractSWUPK Cons){
         switch (Cons.product){
             case Contract.P_PK: {
                ArrayList<Mining> all_actors = World.tbulletin.my_minings;
                double[] all_costs = all_actors.stream().mapToDouble(a -> a.cost).toArray();
                int[] all_ids = all_actors.stream().mapToInt(a -> a.id).toArray();
                int m = DMath.Arithmetic.max(all_costs);
                int my = DMath.Arithmetic.indexOf(all_ids, Cons.getPk_seller().id);
                return all_costs[my]/all_costs[m];
            }
            case Contract.P_SWU: {
                ArrayList<Enrichment> all_actors = World.tbulletin.my_enrichments;
                double[] all_costs = all_actors.stream().mapToDouble(a -> a.cost).toArray();
                int[] all_ids = all_actors.stream().mapToInt(a -> a.id).toArray();
                int m = DMath.Arithmetic.max(all_costs);
                int my = DMath.Arithmetic.indexOf(all_ids, Cons.getEn_seller().id);
                return all_costs[my]/all_costs[m];
            }
            default: throw new Error("Undefunied priduct");
         }
     }
    
    public double getSupplierShare(Contract Cons){
        HashMap<Integer, HashMap<String, Double>> map;
        switch (Cons.product){
            case Contract.P_MOX:
                map = Cons.get_seller().stats.mox;
                break;
            case Contract.P_OUP: map = Cons.get_seller().stats.eup;
                break;
            case Contract.P_REMIX: return 0.5;
            default: throw new Error("This type not expected here");
        }
        double value = map.get(Cons.year).get(Cons.get_seller().getName());
        return value/Cons.get_buyer().get_sum_neeeds();
    }
    
}
    
